@@ -20,18 +20,19 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-import cgen
-import numpy as np
 import re
-from pymbolic.parser import Parser as ExpressionParserBase
-from pymbolic.mapper import CombineMapper
-import pymbolic.primitives as p
-from pymbolic.mapper.c_code import CCodeMapper as CCodeMapperBase
 from sys import intern
-
+from typing import ClassVar, Dict, List, Tuple
 from warnings import warn
 
+import numpy as np
+
+import cgen
+import pymbolic.primitives as p
 import pytools.lex
+from pymbolic.mapper import CombineMapper
+from pymbolic.mapper.c_code import CCodeMapper as CCodeMapperBase
+from pymbolic.parser import Parser as ExpressionParserBase
 
 
 class TranslatorWarning(UserWarning):
@@ -133,7 +134,7 @@ def simplify_typed_literal(expr):
 class FortranExpressionParser(ExpressionParserBase):
     # FIXME double/single prec literals
 
-    lex_table = [
+    lex_table: ClassVar[List[Tuple[str, str]]] = [
         (_less_than, pytools.lex.RE(r"\.lt\.", re.I)),
         (_greater_than, pytools.lex.RE(r"\.gt\.", re.I)),
         (_less_equal, pytools.lex.RE(r"\.le\.", re.I)),
@@ -144,7 +145,7 @@ class FortranExpressionParser(ExpressionParserBase):
         (_not, pytools.lex.RE(r"\.not\.", re.I)),
         (_and, pytools.lex.RE(r"\.and\.", re.I)),
         (_or, pytools.lex.RE(r"\.or\.", re.I)),
-        ] + ExpressionParserBase.lex_table
+        *ExpressionParserBase.lex_table]
 
     def __init__(self, tree_walker):
         self.tree_walker = tree_walker
@@ -154,8 +155,7 @@ class FortranExpressionParser(ExpressionParserBase):
     def parse_terminal(self, pstate):
         scope = self.tree_walker.scope_stack[-1]
 
-        from pymbolic.parser import (
-            _identifier, _openpar, _closepar, _float)
+        from pymbolic.parser import _closepar, _float, _identifier, _openpar
 
         next_tag = pstate.next_tag()
         if next_tag is _float:
@@ -211,7 +211,7 @@ class FortranExpressionParser(ExpressionParserBase):
             return ExpressionParserBase.parse_terminal(
                     self, pstate)
 
-    COMP_MAP = {
+    COMP_MAP: ClassVar[Dict[str, str]] = {
             _less_than: "<",
             _less_equal: "<=",
             _greater_than: ">",
@@ -221,8 +221,8 @@ class FortranExpressionParser(ExpressionParserBase):
             }
 
     def parse_prefix(self, pstate, min_precedence=0):
-        from pymbolic.parser import _PREC_UNARY
         import pymbolic.primitives as primitives
+        from pymbolic.parser import _PREC_UNARY
 
         pstate.expect_not_end()
 
@@ -235,10 +235,13 @@ class FortranExpressionParser(ExpressionParserBase):
 
     def parse_postfix(self, pstate, min_precedence, left_exp):
         from pymbolic.parser import (
-                _PREC_CALL, _PREC_COMPARISON, _openpar,
-                _PREC_LOGICAL_OR, _PREC_LOGICAL_AND)
-        from pymbolic.primitives import (
-                Comparison, LogicalAnd, LogicalOr)
+            _PREC_CALL,
+            _PREC_COMPARISON,
+            _PREC_LOGICAL_AND,
+            _PREC_LOGICAL_OR,
+            _openpar,
+        )
+        from pymbolic.primitives import Comparison, LogicalAnd, LogicalOr
 
         next_tag = pstate.next_tag()
         if next_tag is _openpar and _PREC_CALL > min_precedence:
@@ -351,7 +354,7 @@ class ComplexCCodeMapper(CCodeMapperBase):
             complexes = [child for child in expr.children
                     if "c" == self.infer_type(child).kind]
 
-            from pymbolic.mapper.stringifier import PREC_SUM, PREC_NONE
+            from pymbolic.mapper.stringifier import PREC_NONE, PREC_SUM
             real_sum = self.join_rec(" + ", reals, PREC_SUM)
 
             if len(complexes) == 1:
@@ -387,7 +390,7 @@ class ComplexCCodeMapper(CCodeMapperBase):
             complexes = [child for child in expr.children
                     if "c" == self.infer_type(child).kind]
 
-            from pymbolic.mapper.stringifier import PREC_PRODUCT, PREC_NONE
+            from pymbolic.mapper.stringifier import PREC_NONE, PREC_PRODUCT
             real_prd = self.join_rec("*", reals, PREC_PRODUCT)
 
             if len(complexes) == 1:
@@ -489,7 +492,7 @@ class CCodeMapper(ComplexCCodeMapper):
         if isinstance(idx, tuple) and len(idx) == 1:
             idx, = idx
 
-        from pymbolic.mapper.stringifier import PREC_NONE, PREC_CALL
+        from pymbolic.mapper.stringifier import PREC_CALL, PREC_NONE
         return self.parenthesize_if_needed(
                 self.format("%s[%s%s]",
                     self.scope.translate_var_name(expr.aggregate.name),
@@ -621,7 +624,7 @@ class Scope:
 
             if self.implicit_types is None:
                 raise TranslationError(
-                        "no type for '%s' found in implict none routine"
+                        "no type for '%s' found in implicit none routine"
                         % name)
 
             return self.implicit_types.get(name[0], np.dtype(np.int32))
@@ -944,7 +947,7 @@ class F2CLTranslator(FTreeWalkerBase):
 
             if shape is not None:
                 dim_stmt = cgen.Statement(
-                    'dimension \"fortran\" {}[{}]'.format(
+                    'dimension "fortran" {}[{}]'.format(
                         scope.translate_var_name(name),
                         ", ".join(gen_shape(s) for s in shape)
                         ))
@@ -1020,7 +1023,7 @@ class F2CLTranslator(FTreeWalkerBase):
         body = self.map_statement_list(node.content)
 
         pre_func_decl, in_func_decl = self.get_declarations()
-        body = in_func_decl + [cgen.Line()] + body
+        body = [*in_func_decl, cgen.Line(), *body]
 
         if isinstance(body[-1], cgen.Statement) and body[-1].text == "return":
             body.pop()
@@ -1049,7 +1052,7 @@ class F2CLTranslator(FTreeWalkerBase):
 
         self.scope_stack.pop()
         if pre_func_decl:
-            return pre_func_decl + [cgen.Line(), result]
+            return [*pre_func_decl, cgen.Line(), result]
         else:
             return result
 
@@ -1076,7 +1079,7 @@ class F2CLTranslator(FTreeWalkerBase):
     def map_Equivalence(self, node):
         raise NotImplementedError("equivalence")
 
-    TYPE_MAP = {
+    TYPE_MAP: ClassVar[Dict[Tuple[str, str], np.generic]] = {
             ("real", "4"): np.float32,
             ("real", "8"): np.float64,
             ("real", "16"): np.float128,
@@ -1093,7 +1096,7 @@ class F2CLTranslator(FTreeWalkerBase):
     def dtype_from_stmt(self, stmt):
         length, kind = stmt.selector
         assert not kind
-        return np.dtype(self.TYPE_MAP[(type(stmt).__name__.lower(), length)])
+        return np.dtype(self.TYPE_MAP[type(stmt).__name__.lower(), length])
 
     def map_type_decl(self, node):
         scope = self.scope_stack[-1]
@@ -1201,8 +1204,8 @@ class F2CLTranslator(FTreeWalkerBase):
         raise NotImplementedError("save")
 
     def map_Line(self, node):
-        #from warnings import warn
-        #warn("Encountered a 'line': %s" % node)
+        # from warnings import warn
+        # warn("Encountered a 'line': %s" % node)
         raise NotImplementedError
 
     def map_Program(self, node):
@@ -1385,9 +1388,9 @@ def f2cl(source, free_form=False, strict=True,
         if isinstance(entry, cgen.FunctionBody):
             func_decls.append(entry.fdecl)
 
-    mod = cgen.Module(func_decls + [cgen.Line()] + source)
+    mod = cgen.Module([*func_decls, cgen.Line(), *source])
 
-    #open("pre-cnd.cl", "w").write(str(mod))
+    # open("pre-cnd.cl", "w").write(str(mod))
 
     from cnd import transform_cl
     str_mod = transform_cl(str(mod))
