@@ -1,12 +1,15 @@
+import numpy as np
+
 import pyopencl as cl
 import pyopencl.array as cl_array
 from pyopencl.tools import SVMAllocator, SVMPool
-import numpy as np
+
 
 n = 50000
-a = np.random.rand(n).astype(np.float32)
-b = np.random.rand(n).astype(np.float32)
 
+rng = np.random.default_rng()
+a = rng.random(n, dtype=np.float32)
+b = rng.random(n, dtype=np.float32)
 
 ctx = cl.create_some_context()
 queue = cl.CommandQueue(ctx)
@@ -30,4 +33,5 @@ prg = cl.Program(ctx, """
 knl = prg.sum
 knl(queue, a.shape, None, a_dev.data, b_dev.data, dest_dev.data)
 
-np.testing.assert_allclose(dest_dev.get(), (a_dev+b_dev).get())
+print(np.linalg.norm((dest_dev - (a_dev + b_dev)).get()))
+assert np.allclose(dest_dev.get(), (a_dev + b_dev).get())
