@@ -1,18 +1,24 @@
+import numpy as np
+import numpy.linalg as la
+
 import pyopencl as cl
 import pyopencl.array as cl_array
-import numpy
-import numpy.linalg as la
+from pyopencl.elementwise import ElementwiseKernel
+
 
 ctx = cl.create_some_context()
 queue = cl.CommandQueue(ctx)
 
 n = 10
-a_gpu = cl_array.to_device(queue,
-        (numpy.random.randn(n) + 1j*numpy.random.randn(n)).astype(numpy.complex64))
-b_gpu = cl_array.to_device(queue,
-        (numpy.random.randn(n) + 1j*numpy.random.randn(n)).astype(numpy.complex64))
 
-from pyopencl.elementwise import ElementwiseKernel
+rng = np.random.default_rng()
+a_gpu = cl_array.to_device(queue,
+        rng.standard_normal(n, dtype=np.float32)
+        + 1j*rng.standard_normal(n, dtype=np.float32))
+b_gpu = cl_array.to_device(queue,
+        rng.standard_normal(n, dtype=np.float32)
+        + 1j*rng.standard_normal(n, dtype=np.float32))
+
 complex_prod = ElementwiseKernel(ctx,
         "float a, "
         "cfloat_t *x, "
@@ -39,7 +45,7 @@ real_part = ElementwiseKernel(ctx,
 c_gpu = cl_array.empty_like(a_gpu)
 complex_prod(5, a_gpu, b_gpu, c_gpu)
 
-c_gpu_real = cl_array.empty(queue, len(a_gpu), dtype=numpy.float32)
+c_gpu_real = cl_array.empty(queue, len(a_gpu), dtype=np.float32)
 real_part(c_gpu, c_gpu_real)
 print(c_gpu.get().real - c_gpu_real.get())
 
