@@ -1,3 +1,6 @@
+from __future__ import annotations
+
+
 __copyright__ = "Copyright (C) 2009-15 Andreas Kloeckner"
 
 __license__ = """
@@ -20,16 +23,18 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
+import logging
 from sys import intern
+from typing import Any, Sequence
 from warnings import warn
-from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
-
-from pyopencl.version import VERSION, VERSION_STATUS, VERSION_TEXT  # noqa: F401
 
 # must import, otherwise dtype registry will not be fully populated
-import pyopencl.cltypes  # noqa: F401
+import pyopencl.cltypes
+from pyopencl.version import VERSION, VERSION_STATUS, VERSION_TEXT  # noqa: F401
+
+
+__version__ = VERSION_TEXT
 
-import logging
 logger = logging.getLogger(__name__)
 
 # This supports ocl-icd find shipped OpenCL ICDs, cf.
@@ -37,6 +42,8 @@ logger = logging.getLogger(__name__)
 # via
 # https://github.com/inducer/pyopencl/blob/0b3d0ef92497e6838eea300b974f385f94cb5100/scripts/build-wheels.sh#L43-L44
 import os
+
+
 os.environ["PYOPENCL_HOME"] = os.path.dirname(os.path.abspath(__file__))
 
 try:
@@ -168,6 +175,7 @@ from pyopencl._cl import (  # noqa: F401
         Pipe,
         )
 
+
 try:
     from pyopencl._cl import DeviceTopologyAmd  # noqa: F401
     from pyopencl._cl import enqueue_copy_buffer_p2p_amd  # noqa: F401
@@ -176,46 +184,24 @@ except ImportError:
 
 if not _PYPY:
     # FIXME: Add back to default set when pypy support catches up
-    from pyopencl._cl import (  # noqa: F401
-        enqueue_map_buffer,
-        enqueue_map_image,
-        )
+    from pyopencl._cl import enqueue_map_buffer  # noqa: F401
+    from pyopencl._cl import enqueue_map_image  # noqa: F401
 
 if get_cl_header_version() >= (1, 1):
-    from pyopencl._cl import (  # noqa: F401
-        UserEvent,
-        )
+    from pyopencl._cl import UserEvent  # noqa: F401
 if get_cl_header_version() >= (1, 2):
+    from pyopencl._cl import ImageDescriptor
     from pyopencl._cl import (  # noqa: F401
-        _enqueue_marker_with_wait_list,
-        _enqueue_barrier_with_wait_list,
-
-        unload_platform_compiler,
-
-
-        enqueue_migrate_mem_objects,
-        _enqueue_fill_buffer,
-        enqueue_fill_image,
-
-        ImageDescriptor,
-        )
+        _enqueue_barrier_with_wait_list, _enqueue_fill_buffer,
+        _enqueue_marker_with_wait_list, enqueue_fill_image,
+        enqueue_migrate_mem_objects, unload_platform_compiler)
 
 if get_cl_header_version() >= (2, 0):
-    from pyopencl._cl import (  # noqa: F401
-        SVMPointer,
-        SVM,
-        SVMAllocation,
-        )
+    from pyopencl._cl import SVM, SVMAllocation, SVMPointer
 
 if _cl.have_gl():
     from pyopencl._cl import (  # noqa: F401
-        gl_object_type,
-        gl_texture_info,
-
-        GLBuffer,
-        GLRenderBuffer,
-        GLTexture,
-        )
+        GLBuffer, GLRenderBuffer, GLTexture, gl_object_type, gl_texture_info)
 
     try:
         from pyopencl._cl import get_apple_cgl_share_group  # noqa: F401
@@ -223,15 +209,14 @@ if _cl.have_gl():
         pass
 
     try:
-        from pyopencl._cl import (  # noqa: F401
-            enqueue_acquire_gl_objects,
-            enqueue_release_gl_objects,
-        )
+        from pyopencl._cl import enqueue_acquire_gl_objects  # noqa: F401
+        from pyopencl._cl import enqueue_release_gl_objects  # noqa: F401
     except ImportError:
         pass
 
 import inspect as _inspect
 
+
 CONSTANT_CLASSES = tuple(
         getattr(_cl, name) for name in dir(_cl)
         if _inspect.isclass(getattr(_cl, name))
@@ -266,8 +251,9 @@ class CommandQueueUsedAfterExit(UserWarning):
     pass
 
 
-def compiler_output(text):
-    if int(os.environ.get("PYOPENCL_COMPILER_OUTPUT", "0")):
+def compiler_output(text: str) -> None:
+    from pytools import strtobool
+    if strtobool(os.environ.get("PYOPENCL_COMPILER_OUTPUT", "False")):
         warn(text, CompilerWarning, stacklevel=3)
     else:
         warn("Non-empty compiler output encountered. Set the "
@@ -279,21 +265,21 @@ def compiler_output(text):
 
 # {{{ find pyopencl shipped source code
 
-def _find_pyopencl_include_path():
-    from os.path import join, abspath, dirname, exists
-    try:
-        # Try to find the include path in the same directory as this file
-        include_path = join(abspath(dirname(__file__)), "cl")
+def _find_pyopencl_include_path() -> str:
+    from os.path import abspath, dirname, exists, join
+
+    # Try to find the include path in the same directory as this file
+    include_path = join(abspath(dirname(__file__)), "cl")
+    if not exists(include_path):
+        try:
+            # NOTE: only available in Python >=3.9
+            from importlib.resources import files
+        except ImportError:
+            from importlib_resources import files  # type: ignore[no-redef]
+
+        include_path = str(files("pyopencl") / "cl")
         if not exists(include_path):
-            raise OSError("unable to find pyopencl include path")
-    except Exception:
-        # Try to find the resource with pkg_resources (the recommended
-        # setuptools approach). This is very slow.
-        from pkg_resources import Requirement, resource_filename
-        include_path = resource_filename(
-                Requirement.parse("pyopencl"), "pyopencl/cl")
-        if not exists(include_path):
-            raise OSError("unable to find pyopencl include path")
+            raise OSError("Unable to find PyOpenCL include path")
 
     # Quote the path if it contains a space and is not quoted already.
     # See https://github.com/inducer/pyopencl/issues/250 for discussion.
@@ -310,9 +296,6 @@ def _find_pyopencl_include_path():
 def _split_options_if_necessary(options):
     if isinstance(options, str):
         import shlex
-        # shlex.split takes unicode (py3 str) on py3
-        if isinstance(options, bytes):
-            options = options.decode("utf-8")
 
         options = shlex.split(options)
 
@@ -362,13 +345,16 @@ def _options_to_bytestring(options):
 
 # {{{ Program (wrapper around _Program, adds caching support)
 
-_PYOPENCL_NO_CACHE: bool = "PYOPENCL_NO_CACHE" in os.environ
+from pytools import strtobool
 
-_DEFAULT_BUILD_OPTIONS: List[str] = []
-_DEFAULT_INCLUDE_OPTIONS: List[str] = ["-I", _find_pyopencl_include_path()]
+
+_PYOPENCL_NO_CACHE = strtobool(os.environ.get("PYOPENCL_NO_CACHE", "false"))
+
+_DEFAULT_BUILD_OPTIONS: list[str] = []
+_DEFAULT_INCLUDE_OPTIONS: list[str] = ["-I", _find_pyopencl_include_path()]
 
 # map of platform.name to build options list
-_PLAT_BUILD_OPTIONS: Dict[str, List[str]] = {
+_PLAT_BUILD_OPTIONS: dict[str, list[str]] = {
         "Oclgrind": ["-D", "PYOPENCL_USING_OCLGRIND"],
         }
 
@@ -440,10 +426,7 @@ class Program:
         return self._get_prg().get_build_info(*args, **kwargs)
 
     def all_kernels(self):
-        result = self._get_prg().all_kernels()
-        for knl in result:
-            knl._setup(self)
-        return result
+        return self._get_prg().all_kernels()
 
     @property
     def int_ptr(self):
@@ -460,22 +443,22 @@ class Program:
             knl = Kernel(self, attr)
             # Nvidia does not raise errors even for invalid names,
             # but this will give an error if the kernel is invalid.
-            knl.num_args
-            knl._source = getattr(self, "_source", None)
+            knl.num_args  # noqa: B018
 
             if self._build_duration_info is not None:
-                build_descr, was_cached, duration = self._build_duration_info
+                build_descr, _was_cached, duration = self._build_duration_info
                 if duration > 0.2:
-                    logger.info("build program: kernel '%s' was part of a "
-                            "lengthy %s (%.2f s)" % (attr, build_descr, duration))
+                    logger.info(
+                        "build program: kernel '%s' was part of a "
+                        "lengthy %s (%.2f s)", attr, build_descr, duration)
 
                 # don't whine about build times more than once.
                 self._build_duration_info = None
 
             return knl
-        except LogicError:
+        except LogicError as err:
             raise AttributeError("'%s' was not found as a program "
-                    "info attribute or as a kernel name" % attr)
+                    "info attribute or as a kernel name" % attr) from err
 
     # {{{ build
 
@@ -510,8 +493,16 @@ class Program:
             cache_dir = getattr(self._context, "cache_dir", None)
 
         build_descr = None
-        if _PYOPENCL_NO_CACHE and self._prg is None:
-            build_descr = "uncached source build (cache disabled by user)"
+        from pyopencl.characterize import has_src_build_cache
+
+        if (
+                (_PYOPENCL_NO_CACHE or has_src_build_cache(self._context.devices[0]))
+                and self._prg is None):
+            if _PYOPENCL_NO_CACHE:
+                build_descr = "uncached source build (cache disabled by user)"
+            else:
+                build_descr = "uncached source build (assuming cached by ICD)"
+
             self._prg = _cl._Program(self._context, self._source)
 
         from time import time
@@ -554,7 +545,7 @@ class Program:
     def _build_and_catch_errors(self, build_func, options_bytes, source=None):
         try:
             return build_func()
-        except _cl.RuntimeError as e:
+        except RuntimeError as e:
             msg = str(e)
             if options_bytes:
                 msg = msg + "\n(options: %s)" % options_bytes.decode("utf-8")
@@ -572,7 +563,7 @@ class Program:
             code = e.code
             routine = e.routine
 
-            err = _cl.RuntimeError(
+            err = RuntimeError(
                     _cl._ErrorRecord(
                         msg=msg,
                         code=code,
@@ -675,18 +666,6 @@ def _add_functionality():
 
     # {{{ Context
 
-    context_old_init = Context.__init__
-
-    def context_init(self, devices, properties, dev_type, cache_dir=None):
-        if cache_dir is not None:
-            warn("The 'cache_dir' argument to the Context constructor "
-                "is deprecated and no longer has an effect. "
-                "It was removed because it only applied to the wrapper "
-                "object and not the context itself, leading to inconsistencies.",
-                DeprecationWarning, stacklevel=2)
-
-        context_old_init(self, devices, properties, dev_type)
-
     def context_repr(self):
         return "<pyopencl.Context at 0x{:x} on {}>".format(self.int_ptr,
                 ", ".join(repr(dev) for dev in self.devices))
@@ -789,9 +768,9 @@ def _add_functionality():
 
             try:
                 inf_attr = getattr(info_cls, name.upper())
-            except AttributeError:
+            except AttributeError as err:
                 raise AttributeError("%s has no attribute '%s'"
-                        % (type(self), name))
+                        % (type(self), name)) from err
             else:
                 return self.event.get_profiling_info(inf_attr)
 
@@ -801,31 +780,9 @@ def _add_functionality():
 
     # {{{ Kernel
 
-    kernel_old_init = Kernel.__init__
     kernel_old_get_info = Kernel.get_info
     kernel_old_get_work_group_info = Kernel.get_work_group_info
 
-    def kernel_init(self, prg, name):
-        if not isinstance(prg, _cl._Program):
-            prg = prg._get_prg()
-
-        kernel_old_init(self, prg, name)
-
-        self._setup(prg)
-
-    def kernel__setup(self, prg):
-        self._source = getattr(prg, "_source", None)
-
-        from pyopencl.invoker import generate_enqueue_and_set_args
-        self._enqueue, self._set_args = generate_enqueue_and_set_args(
-                self.function_name, self.num_args, self.num_args,
-                None,
-                warn_about_arg_count_bug=None,
-                work_around_arg_count_bug=None, devs=self.context.devices)
-
-        self._wg_info_cache = {}
-        return self
-
     def kernel_set_arg_types(self, arg_types):
         arg_types = tuple(arg_types)
 
@@ -854,52 +811,31 @@ def _add_functionality():
         # }}}
 
         from pyopencl.invoker import generate_enqueue_and_set_args
-        enqueue, my_set_args = \
-                generate_enqueue_and_set_args(
+        self._set_enqueue_and_set_args(
+                *generate_enqueue_and_set_args(
                         self.function_name,
                         len(arg_types), self.num_args,
                         arg_types,
                         warn_about_arg_count_bug=warn_about_arg_count_bug,
                         work_around_arg_count_bug=work_around_arg_count_bug,
-                        devs=self.context.devices)
-
-        # Make ourselves a kernel-specific class, so that we're able to override
-        # __call__. Inspired by https://stackoverflow.com/a/38541437
-        class KernelWithCustomEnqueue(type(self)):
-            __call__ = enqueue
-            set_args = my_set_args
-
-        try:
-            self.__class__ = KernelWithCustomEnqueue
-        except TypeError:
-            # __class__ assignment may not work in all cases, due to differing
-            # object layouts. Fall back to bouncing through kernel_call below.
-            self._enqueue = enqueue
-            self.set_args = my_set_args
+                        devs=self.context.devices))
 
     def kernel_get_work_group_info(self, param, device):
+        try:
+            wg_info_cache = self._wg_info_cache
+        except AttributeError:
+            wg_info_cache = self._wg_info_cache = {}
+
         cache_key = (param, device.int_ptr)
         try:
-            return self._wg_info_cache[cache_key]
+            return wg_info_cache[cache_key]
         except KeyError:
             pass
 
         result = kernel_old_get_work_group_info(self, param, device)
-        self._wg_info_cache[cache_key] = result
+        wg_info_cache[cache_key] = result
         return result
 
-    def kernel_set_args(self, *args, **kwargs):
-        # Need to duplicate the 'self' argument for dynamically generated  method
-        return self._set_args(self, *args, **kwargs)
-
-    def kernel_call(self, queue, global_size, local_size, *args, **kwargs):
-        # __call__ can't be overridden directly, so we need this
-        # trampoline hack.
-
-        # Note: This is only used for the generic __call__, before
-        # kernel_set_scalar_arg_dtypes is called.
-        return self._enqueue(self, queue, global_size, local_size, *args, **kwargs)
-
     def kernel_capture_call(self, output_file, queue, global_size, local_size,
             *args, **kwargs):
         from pyopencl.capture_call import capture_kernel_call
@@ -914,16 +850,12 @@ def _add_functionality():
         else:
             return val
 
-    Kernel.__init__ = kernel_init
-    Kernel._setup = kernel__setup
     Kernel.get_work_group_info = kernel_get_work_group_info
 
     # FIXME: Possibly deprecate this version
     Kernel.set_scalar_arg_dtypes = kernel_set_arg_types
     Kernel.set_arg_types = kernel_set_arg_types
 
-    Kernel.set_args = kernel_set_args
-    Kernel.__call__ = kernel_call
     Kernel.capture_call = kernel_capture_call
     Kernel.get_info = kernel_get_info
 
@@ -957,10 +889,30 @@ def _add_functionality():
 
     # {{{ Image
 
-    image_old_init = Image.__init__
+    def image_init(
+                self, context, flags, format, shape=None, pitches=None,
+                hostbuf=None, is_array=False, buffer=None, *,
+                desc: ImageDescriptor | None = None,
+                _through_create_image: bool = False,
+            ) -> None:
+        if hostbuf is not None and not \
+                (flags & (mem_flags.USE_HOST_PTR | mem_flags.COPY_HOST_PTR)):
+            warn("'hostbuf' was passed, but no memory flags to make use of it.",
+                 stacklevel=2)
 
-    def image_init(self, context, flags, format, shape=None, pitches=None,
-            hostbuf=None, is_array=False, buffer=None):
+        if desc is not None:
+            if shape is not None:
+                raise TypeError("shape may not be passed when using descriptor")
+            if pitches is not None:
+                raise TypeError("pitches may not be passed when using descriptor")
+            if is_array:
+                raise TypeError("is_array may not be passed when using descriptor")
+            if buffer is not None:
+                raise TypeError("is_array may not be passed when using descriptor")
+
+            Image._custom_init(self, context, flags, format, desc, hostbuf)
+
+            return
 
         if shape is None and hostbuf is None:
             raise Error("'shape' must be passed if 'hostbuf' is not given")
@@ -968,15 +920,16 @@ def _add_functionality():
         if shape is None and hostbuf is not None:
             shape = hostbuf.shape
 
-        if hostbuf is not None and not \
-                (flags & (mem_flags.USE_HOST_PTR | mem_flags.COPY_HOST_PTR)):
-            warn("'hostbuf' was passed, but no memory flags to make use of it.",
-                 stacklevel=2)
-
         if hostbuf is None and pitches is not None:
             raise Error("'pitches' may only be given if 'hostbuf' is given")
 
         if context._get_cl_version() >= (1, 2) and get_cl_header_version() >= (1, 2):
+            if not _through_create_image:
+                warn("Non-descriptor Image constructor called. "
+                     "This will stop working in 2026. "
+                     "Use create_image instead (with the same arguments).",
+                     DeprecationWarning, stacklevel=2)
+
             if buffer is not None and is_array:
                 raise ValueError(
                         "'buffer' and 'is_array' are mutually exclusive")
@@ -1010,7 +963,8 @@ def _add_functionality():
             else:
                 raise ValueError("images cannot have more than three dimensions")
 
-            desc = ImageDescriptor()
+            desc = ImageDescriptor() \
+                # pylint: disable=possibly-used-before-assignment
 
             desc.image_type = image_type
             desc.shape = shape  # also sets desc.array_size
@@ -1024,7 +978,7 @@ def _add_functionality():
             desc.num_samples = 0  # per CL 1.2 spec
             desc.buffer = buffer
 
-            image_old_init(self, context, flags, format, desc, hostbuf)
+            Image._custom_init(self, context, flags, format, desc, hostbuf)
         else:
             # legacy init for CL 1.1 and older
             if is_array:
@@ -1038,7 +992,7 @@ def _add_functionality():
             if buffer is not None:
                 raise TypeError("'buffer' argument is not supported for CL < 1.2")
 
-            image_old_init(self, context, flags, format, shape,
+            Image._custom_init(self, context, flags, format, shape,
                     pitches, hostbuf)
 
     class _ImageInfoGetter:
@@ -1052,9 +1006,9 @@ def _add_functionality():
         def __getattr__(self, name):
             try:
                 inf_attr = getattr(_cl.image_info, name.upper())
-            except AttributeError:
+            except AttributeError as err:
                 raise AttributeError("%s has no attribute '%s'"
-                        % (type(self), name))
+                        % (type(self), name)) from err
             else:
                 return self.event.get_image_info(inf_attr)
 
@@ -1077,7 +1031,7 @@ def _add_functionality():
     def error_str(self):
         val = self.what
         try:
-            val.routine
+            val.routine  # noqa: B018
         except AttributeError:
             return str(val)
         else:
@@ -1173,8 +1127,8 @@ def _add_functionality():
             """
 
     def svmptr_map(self, queue: CommandQueue, *, flags: int, is_blocking: bool =
-                   True, wait_for: Optional[Sequence[Event]] = None,
-                   size: Optional[Event] = None) -> "SVMMap":
+                   True, wait_for: Sequence[Event] | None = None,
+                   size: Event | None = None) -> SVMMap:
         """
         :arg is_blocking: If *False*, subsequent code must wait on
             :attr:`SVMMap.event` in the returned object before accessing the
@@ -1192,8 +1146,8 @@ def _add_functionality():
                                     size=size))
 
     def svmptr_map_ro(self, queue: CommandQueue, *, is_blocking: bool = True,
-                      wait_for: Optional[Sequence[Event]] = None,
-                      size: Optional[int] = None) -> "SVMMap":
+                      wait_for: Sequence[Event] | None = None,
+                      size: int | None = None) -> SVMMap:
         """Like :meth:`map`, but with *flags* set for a read-only map.
         """
 
@@ -1201,8 +1155,8 @@ def _add_functionality():
                 is_blocking=is_blocking, wait_for=wait_for, size=size)
 
     def svmptr_map_rw(self, queue: CommandQueue, *, is_blocking: bool = True,
-                      wait_for: Optional[Sequence[Event]] = None,
-                      size: Optional[int] = None) -> "SVMMap":
+                      wait_for: Sequence[Event] | None = None,
+                      size: int | None = None) -> SVMMap:
         """Like :meth:`map`, but with *flags* set for a read-only map.
         """
 
@@ -1212,8 +1166,8 @@ def _add_functionality():
     def svmptr__enqueue_unmap(self, queue, wait_for=None):
         return _cl._enqueue_svm_unmap(queue, self, wait_for)
 
-    def svmptr_as_buffer(self, ctx: Context, *, flags: Optional[int] = None,
-                         size: Optional[int] = None) -> Buffer:
+    def svmptr_as_buffer(self, ctx: Context, *, flags: int | None = None,
+                         size: int | None = None) -> Buffer:
         """
         :arg ctx: a :class:`Context`
         :arg flags: a combination of :class:`pyopencl.map_flags`, defaults to
@@ -1381,14 +1335,6 @@ def _add_functionality():
 
     # }}}
 
-    if get_cl_header_version() >= (2, 0):
-        svm_old_init = SVM.__init__
-
-    def svm_init(self, mem):
-        svm_old_init(self, mem)
-
-        self.mem = mem
-
     def svm_map(self, queue, flags, is_blocking=True, wait_for=None):
         """
         :arg is_blocking: If *False*, subsequent code must wait on
@@ -1425,7 +1371,6 @@ def _add_functionality():
         return _cl._enqueue_svm_unmap(queue, self, wait_for)
 
     if get_cl_header_version() >= (2, 0):
-        SVM.__init__ = svm_init
         SVM.map = svm_map
         SVM.map_ro = svm_map_ro
         SVM.map_rw = svm_map_rw
@@ -1553,9 +1498,62 @@ if get_cl_header_version() >= (2, 0):
 # }}}
 
 
+# {{{ create_image
+
+def create_image(context, flags, format, shape=None, pitches=None,
+        hostbuf=None, is_array=False, buffer=None) -> Image:
+    """
+    See :class:`mem_flags` for values of *flags*.
+    *shape* is a 2- or 3-tuple. *format* is an instance of :class:`ImageFormat`.
+    *pitches* is a 1-tuple for 2D images and a 2-tuple for 3D images, indicating
+    the distance in bytes from one scan line to the next, and from one 2D image
+    slice to the next.
+
+    If *hostbuf* is given and *shape* is *None*, then *hostbuf.shape* is
+    used as the *shape* parameter.
+
+    :class:`Image` inherits from :class:`MemoryObject`.
+
+    .. note::
+
+        If you want to load images from :class:`numpy.ndarray` instances or read images
+        back into them, be aware that OpenCL images expect the *x* dimension to vary
+        fastest, whereas in the default (C) order of :mod:`numpy` arrays, the last index
+        varies fastest. If your array is arranged in the wrong order in memory,
+        there are two possible fixes for this:
+
+        * Convert the array to Fortran (column-major) order using :func:`numpy.asarray`.
+
+        * Pass *ary.T.copy()* to the image creation function.
+
+    .. versionadded:: 2024.3
+    """
+
+    return Image(context, flags, format, shape=shape, pitches=pitches,
+        hostbuf=hostbuf, is_array=is_array, buffer=buffer,
+        _through_create_image=True)
+
+# }}}
+
+
 # {{{ create_some_context
 
-def create_some_context(interactive=None, answers=None):
+def choose_devices(interactive: bool | None = None,
+                   answers: list[str] | None = None) -> list[Device]:
+    """
+    Choose :class:`Device` instances 'somehow'.
+
+    :arg interactive: If multiple choices for platform and/or device exist,
+        *interactive* is ``True`` (or ``None`` and ``sys.stdin.isatty()``
+        returns ``True``), then the user is queried about which device should be
+        chosen. Otherwise, a device is chosen in an implementation-defined
+        manner.
+    :arg answers: A sequence of strings that will be used to answer the
+        platform/device selection questions.
+
+    :returns: a list of :class:`Device` instances.
+    """
+
     if answers is None:
         if "PYOPENCL_CTX" in os.environ:
             ctx_spec = os.environ["PYOPENCL_CTX"]
@@ -1565,7 +1563,7 @@ def create_some_context(interactive=None, answers=None):
             from pyopencl.tools import get_test_platforms_and_devices
             for _plat, devs in get_test_platforms_and_devices():
                 for dev in devs:
-                    return Context([dev])
+                    return [dev]
 
     if answers is not None:
         pre_provided_answers = answers
@@ -1653,7 +1651,8 @@ def create_some_context(interactive=None, answers=None):
 
     if not devices:
         raise Error("no devices found")
-    elif len(devices) == 1:
+    elif len(devices) == 1 and not answers:
+        cc_print(f"Choosing only available device: {devices[0]}")
         pass
     else:
         if not answers:
@@ -1677,7 +1676,27 @@ def create_some_context(interactive=None, answers=None):
 
     if answers:
         raise RuntimeError("not all provided choices were used by "
-                "create_some_context. (left over: '%s')" % ":".join(answers))
+                "choose_devices. (left over: '%s')" % ":".join(answers))
+
+    return devices
+
+
+def create_some_context(interactive: bool | None = None,
+                        answers: list[str] | None = None) -> Context:
+    """
+    Create a :class:`Context` 'somehow'.
+
+    :arg interactive: If multiple choices for platform and/or device exist,
+        *interactive* is ``True`` (or ``None`` and ``sys.stdin.isatty()``
+        returns ``True``), then the user is queried about which device should be
+        chosen. Otherwise, a device is chosen in an implementation-defined
+        manner.
+    :arg answers: A sequence of strings that will be used to answer the
+        platform/device selection questions.
+
+    :returns: an instance of :class:`Context`.
+    """
+    devices = choose_devices(interactive, answers)
 
     return Context(devices)
 
@@ -2052,9 +2071,9 @@ def enqueue_copy(queue, dest, src, **kwargs):
 # {{{ enqueue_fill
 
 def enqueue_fill(queue: CommandQueue,
-        dest: "Union[MemoryObject, SVMPointer]",
+        dest: MemoryObject | SVMPointer,
         pattern: Any, size: int, *, offset: int = 0,
-        wait_for: Optional[Sequence[Event]] = None) -> Event:
+        wait_for: Sequence[Event] | None = None) -> Event:
     """
     .. versionadded:: 2022.2
     """
@@ -2083,7 +2102,7 @@ DTYPE_TO_CHANNEL_TYPE = {
     np.dtype(np.uint8): channel_type.UNSIGNED_INT8,
     }
 try:
-    np.float16
+    np.float16  # noqa: B018
 except Exception:
     pass
 else:
@@ -2145,7 +2164,7 @@ def image_from_array(ctx, ary, num_channels=None, mode="r", norm_int=False):
     else:
         channel_type = DTYPE_TO_CHANNEL_TYPE[dtype]
 
-    return Image(ctx, mode_flags | mem_flags.COPY_HOST_PTR,
+    return create_image(ctx, mode_flags | mem_flags.COPY_HOST_PTR,
             ImageFormat(img_format, channel_type),
             shape=shape[::-1], pitches=strides[::-1][1:],
             hostbuf=ary)
@@ -2258,12 +2277,12 @@ def svm_empty(ctx, flags, shape, dtype, order="C", alignment=None, queue=None):
         s = 1
         for dim in shape:
             s *= dim
-    except TypeError:
+    except TypeError as err:
         admissible_types = (int, np.integer)
 
         if not isinstance(shape, admissible_types):
             raise TypeError("shape must either be iterable or "
-                    "castable to an integer")
+                    "castable to an integer") from err
         s = shape
         shape = (shape,)
 
@@ -2378,14 +2397,14 @@ def fsvm_empty_like(ctx, ary, alignment=None):
 # }}}
 
 
-_KERNEL_ARG_CLASSES: Tuple[type, ...] = (
+_KERNEL_ARG_CLASSES: tuple[type, ...] = (
         MemoryObjectHolder,
         Sampler,
         CommandQueue,
         LocalMemory,
         )
 if get_cl_header_version() >= (2, 0):
-    _KERNEL_ARG_CLASSES = _KERNEL_ARG_CLASSES + (SVM,)
+    _KERNEL_ARG_CLASSES = (*_KERNEL_ARG_CLASSES, SVM)
 
 
 # vim: foldmethod=marker
