@@ -33,24 +33,23 @@ from dataclasses import dataclass
 from typing import Optional
 
 import numpy as np
+from mako.template import Template
+
+from pytools import memoize, memoize_method
 
 import pyopencl as cl
 import pyopencl.array
-
 from pyopencl.elementwise import ElementwiseKernel
-from pyopencl.scan import ScanTemplate, GenericScanKernel
+from pyopencl.scan import GenericScanKernel, ScanTemplate
 from pyopencl.tools import dtype_to_ctype, get_arg_offset_adjuster_code
 
-from pytools import memoize, memoize_method
-from mako.template import Template
-
 
 # {{{ "extra args" handling utility
 
 def _extract_extra_args_types_values(extra_args):
     if extra_args is None:
         extra_args = []
-    from pyopencl.tools import VectorArg, ScalarArg
+    from pyopencl.tools import ScalarArg, VectorArg
 
     extra_args_types = []
     extra_args_values = []
@@ -469,7 +468,7 @@ class RadixSort:
         scan_ctype, scan_dtype, scan_t_cdecl = \
                 _make_sort_scan_type(context.devices[0], self.bits, self.index_dtype)
 
-        from pyopencl.tools import VectorArg, ScalarArg
+        from pyopencl.tools import ScalarArg, VectorArg
         scan_arguments = (
                 list(self.arguments)
                 + [VectorArg(arg.dtype, "sorted_"+arg.name) for arg in self.arguments
@@ -487,17 +486,17 @@ class RadixSort:
                 get_count_branch(known_bits+"0"),
                 get_count_branch(known_bits+"1")))
 
-        codegen_args = dict(
-                bits=self.bits,
-                key_ctype=dtype_to_ctype(self.key_dtype),
-                key_expr=key_expr,
-                index_ctype=dtype_to_ctype(self.index_dtype),
-                index_type_max=np.iinfo(self.index_dtype).max,
-                padded_bin=_padded_bin,
-                scan_ctype=scan_ctype,
-                sort_arg_names=sort_arg_names,
-                get_count_branch=get_count_branch,
-                )
+        codegen_args = {
+                "bits": self.bits,
+                "key_ctype": dtype_to_ctype(self.key_dtype),
+                "key_expr": key_expr,
+                "index_ctype": dtype_to_ctype(self.index_dtype),
+                "index_type_max": np.iinfo(self.index_dtype).max,
+                "padded_bin": _padded_bin,
+                "scan_ctype": scan_ctype,
+                "sort_arg_names": sort_arg_names,
+                "get_count_branch": get_count_branch,
+                }
 
         preamble = scan_t_cdecl+RADIX_SORT_PREAMBLE_TPL.render(**codegen_args)
         scan_preamble = preamble \
@@ -927,7 +926,7 @@ class ListOfListsBuilder:
     @memoize_method
     def get_count_kernel(self, index_dtype):
         index_ctype = dtype_to_ctype(index_dtype)
-        from pyopencl.tools import VectorArg, OtherArg
+        from pyopencl.tools import OtherArg, VectorArg
         kernel_list_args = [
                 VectorArg(index_dtype, "plb_%s_count" % name)
                 for name, dtype in self.list_names_and_dtypes
@@ -978,15 +977,17 @@ class ListOfListsBuilder:
         knl = getattr(prg, kernel_name)
 
         from pyopencl.tools import get_arg_list_scalar_arg_dtypes
-        knl.set_scalar_arg_dtypes(get_arg_list_scalar_arg_dtypes(
-            kernel_list_args+self.arg_decls) + [index_dtype])
+        knl.set_scalar_arg_dtypes([
+            *get_arg_list_scalar_arg_dtypes([*kernel_list_args, *self.arg_decls]),
+            index_dtype
+            ])
 
         return knl
 
     @memoize_method
     def get_write_kernel(self, index_dtype):
         index_ctype = dtype_to_ctype(index_dtype)
-        from pyopencl.tools import VectorArg, OtherArg
+        from pyopencl.tools import OtherArg, VectorArg
         kernel_list_args = []
         kernel_list_arg_values = ""
         user_list_args = []
@@ -1051,8 +1052,9 @@ class ListOfListsBuilder:
         knl = getattr(prg, kernel_name)
 
         from pyopencl.tools import get_arg_list_scalar_arg_dtypes
-        knl.set_scalar_arg_dtypes(get_arg_list_scalar_arg_dtypes(
-            kernel_list_args+self.arg_decls) + [index_dtype])
+        knl.set_scalar_arg_dtypes([
+            *get_arg_list_scalar_arg_dtypes(kernel_list_args + self.arg_decls),
+            index_dtype])
 
         return knl
 
@@ -1227,13 +1229,14 @@ class ListOfListsBuilder:
             info_record.compressed_indices = cl.array.empty(
                 queue, (n_objects + 1,), index_dtype, allocator=allocator)
             info_record.compressed_indices[0] = 0
-            compress_events[name] = compress_kernel(
+
+            compress_events[name] = compress_kernel(  # pylint: disable=possibly-used-before-assignment
                 info_record.starts,
                 compressed_counts,
                 info_record.nonempty_indices,
                 info_record.compressed_indices,
                 info_record.num_nonempty_lists,
-                wait_for=[count_event] + info_record.compressed_indices.events)
+                wait_for=[count_event, *info_record.compressed_indices.events])
 
             info_record.starts = compressed_counts
 
@@ -1369,7 +1372,7 @@ class KeyValueSorter:
 
     @memoize_method
     def get_kernels(self, key_dtype, value_dtype, starts_dtype):
-        from pyopencl.tools import VectorArg, ScalarArg
+        from pyopencl.tools import ScalarArg, VectorArg
 
         by_target_sorter = RadixSort(
                 self.context, [
