@@ -23,28 +23,33 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-import re
+import logging
 import os
+import re
 import sys
 from dataclasses import dataclass
 from typing import List, Optional, Tuple
 
 import pyopencl._cl as _cl
 
-import logging
+
 logger = logging.getLogger(__name__)
 
 
 import hashlib
+
+
 new_hash = hashlib.md5
 
 
-def _erase_dir(dir):
-    from os import listdir, unlink, rmdir
+def _erase_dir(directory):
+    from os import listdir, rmdir, unlink
     from os.path import join
-    for name in listdir(dir):
-        unlink(join(dir, name))
-    rmdir(dir)
+
+    for name in listdir(directory):
+        unlink(join(directory, name))
+
+    rmdir(directory)
 
 
 def update_checksum(checksum, obj):
@@ -168,7 +173,7 @@ C_INCLUDE_RE = re.compile(r'^\s*\#\s*include\s+[<"](.+)[">]\s*$',
 def get_dependencies(src, include_path):
     result = {}
 
-    from os.path import realpath, join
+    from os.path import join, realpath
 
     def _inner(src):
         for match in C_INCLUDE_RE.finditer(src):
@@ -189,7 +194,7 @@ def get_dependencies(src, include_path):
                     finally:
                         src_file.close()
 
-                    # jrevent infinite recursion if some header file appears to
+                    # prevent infinite recursion if some header file appears to
                     # include itself
                     result[included_file_name] = None
 
@@ -210,7 +215,7 @@ def get_dependencies(src, include_path):
 
     _inner(src)
 
-    result = [(name,) + vals for name, vals in result.items()]
+    result = [(name, *vals) for name, vals in result.items()]
     result.sort()
 
     return result
@@ -263,10 +268,10 @@ def get_cache_key(device, options_bytes, src):
 
 
 def retrieve_from_cache(cache_dir, cache_key):
-    class _InvalidInfoFile(RuntimeError):
+    class _InvalidInfoFileError(RuntimeError):
         pass
 
-    from os.path import join, isdir
+    from os.path import isdir, join
     module_cache_dir = join(cache_dir, cache_key)
     if not isdir(module_cache_dir):
         return None
@@ -287,18 +292,18 @@ def retrieve_from_cache(cache_dir, cache_key):
 
                 try:
                     info_file = open(info_path, "rb")
-                except OSError:
-                    raise _InvalidInfoFile()
+                except OSError as err:
+                    raise _InvalidInfoFileError() from err
 
                 try:
                     try:
                         info = load(info_file)
-                    except EOFError:
-                        raise _InvalidInfoFile()
+                    except EOFError as err:
+                        raise _InvalidInfoFileError() from err
                 finally:
                     info_file.close()
 
-            except _InvalidInfoFile:
+            except _InvalidInfoFileError:
                 mod_cache_dir_m.reset()
                 from warnings import warn
                 warn(
@@ -343,25 +348,22 @@ def _create_built_program_from_source_cached(ctx, src, options_bytes,
     from os.path import join
 
     if cache_dir is None:
-        try:
-            import platformdirs as appdirs
-        except ImportError:
-            import appdirs
+        import platformdirs
 
-        cache_dir = join(appdirs.user_cache_dir("pyopencl", "pyopencl"),
+        # Determine the cache directory in the same way as pytools.PersistentDict,
+        # which PyOpenCL uses for invoker caches.
+        if sys.platform == "darwin" and os.getenv("XDG_CACHE_HOME") is not None:
+            # platformdirs does not handle XDG_CACHE_HOME on macOS
+            # https://github.com/platformdirs/platformdirs/issues/269
+            cache_dir = join(os.getenv("XDG_CACHE_HOME"), "pyopencl")
+        else:
+            cache_dir = platformdirs.user_cache_dir("pyopencl", "pyopencl")
+
+        cache_dir = join(cache_dir,
                 "pyopencl-compiler-cache-v2-py{}".format(
                     ".".join(str(i) for i in sys.version_info)))
 
-    # {{{ ensure cache directory exists
-
-    try:
-        os.makedirs(cache_dir)
-    except OSError as e:
-        from errno import EEXIST
-        if e.errno != EEXIST:
-            raise
-
-    # }}}
+    os.makedirs(cache_dir, exist_ok=True)
 
     if devices is None:
         devices = ctx.devices
@@ -375,13 +377,13 @@ def _create_built_program_from_source_cached(ctx, src, options_bytes,
         cache_result = retrieve_from_cache(cache_dir, cache_key)
 
         if cache_result is None:
-            logger.debug("build program: binary cache miss (key: %s)" % cache_key)
+            logger.debug("build program: binary cache miss (key: %s)", cache_key)
 
             to_be_built_indices.append(i)
             binaries.append(None)
             logs.append(None)
         else:
-            logger.debug("build program: binary cache hit (key: %s)" % cache_key)
+            logger.debug("build program: binary cache hit (key: %s)", cache_key)
 
             binary, log = cache_result
             binaries.append(binary)
@@ -410,8 +412,9 @@ def _create_built_program_from_source_cached(ctx, src, options_bytes,
         src = src + "\n\n__constant int pyopencl_defeat_cache_%s = 0;" % (
                 uuid4().hex)
 
-        logger.debug("build program: start building program from source on %s"
-                % ", ".join(str(devices[i]) for i in to_be_built_indices))
+        logger.debug(
+                "build program: start building program from source on %s",
+                ", ".join(str(devices[i]) for i in to_be_built_indices))
 
         prg = _cl._Program(ctx, src)
         prg.build(options_bytes, [devices[i] for i in to_be_built_indices])
@@ -459,13 +462,11 @@ def _create_built_program_from_source_cached(ctx, src, options_bytes,
                     binary_path = mod_cache_dir_m.sub("binary")
                     source_path = mod_cache_dir_m.sub("source.cl")
 
-                    outf = open(source_path, "wt")
-                    outf.write(src)
-                    outf.close()
+                    with open(source_path, "w") as outf:
+                        outf.write(src)
 
-                    outf = open(binary_path, "wb")
-                    outf.write(binary)
-                    outf.close()
+                    with open(binary_path, "wb") as outf:
+                        outf.write(binary)
 
                     from pickle import dump
                     info_file = open(info_path, "wb")
@@ -504,7 +505,7 @@ def create_built_program_from_source_cached(ctx, src, options_bytes, devices=Non
     except Exception as e:
         from pyopencl import Error
         build_program_failure = (isinstance(e, Error)
-                and e.code == _cl.status_code.BUILD_PROGRAM_FAILURE)  # noqa pylint:disable=no-member
+                and e.code == _cl.status_code.BUILD_PROGRAM_FAILURE)  # pylint:disable=no-member
 
         # Mac error on intel CPU driver: can't build from cached version.
         # If we get a build_program_failure from the cached version then
@@ -513,8 +514,8 @@ def create_built_program_from_source_cached(ctx, src, options_bytes, devices=Non
             raise
 
         if not build_program_failure:
-            from warnings import warn
             from traceback import format_exc
+            from warnings import warn
             warn(
                 "PyOpenCL compiler caching failed with an exception:\n"
                 f"[begin exception]\n{format_exc()}[end exception]",
