@@ -22,14 +22,17 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
+from typing import Any, Tuple
+from warnings import warn
+
 import numpy as np
 
-from warnings import warn
-import pyopencl._cl as _cl
 from pytools.persistent_dict import WriteOncePersistentDict
 from pytools.py_codegen import Indentation, PythonCodeGenerator
-from pyopencl.tools import _NumpyTypesKeyBuilder, VectorArg
+
 import pyopencl as cl
+import pyopencl._cl as _cl
+from pyopencl.tools import VectorArg, _NumpyTypesKeyBuilder
 
 
 # {{{ arg packing helpers
@@ -256,16 +259,17 @@ def _generate_enqueue_and_set_args_module(function_name,
 
     # {{{ generate _enqueue
 
-    enqueue_name = "enqueue_knl_%s" % function_name
+    from pytools import to_identifier
+    enqueue_name = f"enqueue_knl_{to_identifier(function_name)}"
     gen("def %s(%s):"
             % (enqueue_name,
-                ", ".join(
-                    ["self", "queue", "global_size", "local_size"]
-                    + arg_names
-                    + ["global_offset=None",
-                        "g_times_l=False",
-                        "allow_empty_ndrange=False",
-                        "wait_for=None"])))
+                ", ".join([
+                    "self", "queue", "global_size", "local_size",
+                    *arg_names,
+                    "global_offset=None",
+                    "g_times_l=False",
+                    "allow_empty_ndrange=False",
+                    "wait_for=None"])))
 
     with Indentation(gen):
         subgen, wait_for_parts = gen_arg_setting(in_enqueue=True)
@@ -293,7 +297,7 @@ def _generate_enqueue_and_set_args_module(function_name,
 
     gen("")
     gen("def set_args(%s):"
-            % (", ".join(["self"] + arg_names)))
+            % (", ".join(["self", *arg_names])))
 
     with Indentation(gen):
         gen.extend(gen_arg_setting(in_enqueue=False))
@@ -373,10 +377,14 @@ def _check_arg_size(function_name, num_cl_args, arg_types, devs):
 # }}}
 
 
-invoker_cache = WriteOncePersistentDict(
-        "pyopencl-invoker-cache-v41",
-        key_builder=_NumpyTypesKeyBuilder(),
-        in_mem_cache_size=0)
+if not cl._PYOPENCL_NO_CACHE:
+    from pytools.py_codegen import PicklableModule
+    invoker_cache: WriteOncePersistentDict[Any, Tuple[PicklableModule, str]] \
+        = WriteOncePersistentDict(
+            "pyopencl-invoker-cache-v42-nano",
+            key_builder=_NumpyTypesKeyBuilder(),
+            in_mem_cache_size=0,
+            safe_sync=False)
 
 
 def generate_enqueue_and_set_args(function_name,
@@ -400,7 +408,8 @@ def generate_enqueue_and_set_args(function_name,
 
     if not from_cache:
         pmod, enqueue_name = _generate_enqueue_and_set_args_module(*cache_key)
-        invoker_cache.store_if_not_present(cache_key, (pmod, enqueue_name))
+        if not cl._PYOPENCL_NO_CACHE:
+            invoker_cache.store_if_not_present(cache_key, (pmod, enqueue_name))
 
     return (
             pmod.mod_globals[enqueue_name],
