@@ -22,33 +22,32 @@ limitations under the License.
 Derived from code within the Thrust project, https://github.com/NVIDIA/thrust
 """
 
+import logging
 from abc import ABC, abstractmethod
 from dataclasses import dataclass
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 import numpy as np
 
-import pyopencl as cl
-import pyopencl.array
-from pyopencl.tools import (
-        KernelTemplateBase,
-        DtypedArgument,
-        bitlog2,
-        context_dependent_memoize,
-        dtype_to_ctype,
-        get_arg_list_scalar_arg_dtypes,
-        get_arg_offset_adjuster_code,
-        _process_code_for_macro,
-        _NumpyTypesKeyBuilder,
-        )
-
-import pyopencl._mymako as mako
-from pyopencl._cluda import CLUDA_PREAMBLE
-
 from pytools.persistent_dict import WriteOncePersistentDict
 
+import pyopencl as cl
+import pyopencl._mymako as mako
+import pyopencl.array
+from pyopencl._cluda import CLUDA_PREAMBLE
+from pyopencl.tools import (
+    DtypedArgument,
+    KernelTemplateBase,
+    _NumpyTypesKeyBuilder,
+    _process_code_for_macro,
+    bitlog2,
+    context_dependent_memoize,
+    dtype_to_ctype,
+    get_arg_list_scalar_arg_dtypes,
+    get_arg_offset_adjuster_code,
+)
+
 
-import logging
 logger = logging.getLogger(__name__)
 
 
@@ -1056,9 +1055,9 @@ class GenericScanKernelBase(ABC):
         from pyopencl.tools import parse_arg_list
         self.parsed_args = parse_arg_list(arguments)
         from pyopencl.tools import VectorArg
-        self.first_array_idx = [
+        self.first_array_idx = next(
                 i for i, arg in enumerate(self.parsed_args)
-                if isinstance(arg, VectorArg)][0]
+                if isinstance(arg, VectorArg))
 
         self.input_expr = input_expr
 
@@ -1141,10 +1140,15 @@ class GenericScanKernelBase(ABC):
         pass
 
 
-generic_scan_kernel_cache = WriteOncePersistentDict(
-        "pyopencl-generated-scan-kernel-cache-v1",
-        key_builder=_NumpyTypesKeyBuilder(),
-        in_mem_cache_size=0)
+if not cl._PYOPENCL_NO_CACHE:
+    generic_scan_kernel_cache: WriteOncePersistentDict[Any,
+                    Tuple[_GeneratedScanKernelInfo, _GeneratedScanKernelInfo,
+                    _GeneratedFinalUpdateKernelInfo]] = \
+        WriteOncePersistentDict(
+            "pyopencl-generated-scan-kernel-cache-v1",
+            key_builder=_NumpyTypesKeyBuilder(),
+            in_mem_cache_size=0,
+            safe_sync=False)
 
 
 class GenericScanKernel(GenericScanKernelBase):
@@ -1182,7 +1186,7 @@ class GenericScanKernel(GenericScanKernelBase):
                 result = generic_scan_kernel_cache[cache_key]
                 from_cache = True
                 logger.debug(
-                    "cache hit for generated scan kernel '%s'" % self.name_prefix)
+                    "cache hit for generated scan kernel '%s'", self.name_prefix)
                 (
                     self.first_level_scan_gen_info,
                     self.second_level_scan_gen_info,
@@ -1192,14 +1196,15 @@ class GenericScanKernel(GenericScanKernelBase):
 
         if not from_cache:
             logger.debug(
-                    "cache miss for generated scan kernel '%s'" % self.name_prefix)
+                    "cache miss for generated scan kernel '%s'", self.name_prefix)
             self._finish_setup_impl()
 
             result = (self.first_level_scan_gen_info,
                       self.second_level_scan_gen_info,
                       self.final_update_gen_info)
 
-            generic_scan_kernel_cache.store_if_not_present(cache_key, result)
+            if not cl._PYOPENCL_NO_CACHE:
+                generic_scan_kernel_cache.store_if_not_present(cache_key, result)
 
         # Build the kernels.
         self.first_level_scan_info = self.first_level_scan_gen_info.build(
@@ -1319,8 +1324,10 @@ class GenericScanKernel(GenericScanKernelBase):
         # {{{ build second-level scan
 
         from pyopencl.tools import VectorArg
-        second_level_arguments = self.parsed_args + [
-                VectorArg(self.dtype, "interval_sums")]
+        second_level_arguments = [
+            *self.parsed_args,
+            VectorArg(self.dtype, "interval_sums"),
+            ]
 
         second_level_build_kwargs: Dict[str, Optional[str]] = {}
         if self.is_segmented:
@@ -1367,9 +1374,10 @@ class GenericScanKernel(GenericScanKernelBase):
             use_lookbehind_update=self.use_lookbehind_update,
             **self.code_variables))
 
-        update_scalar_arg_dtypes = (
-                get_arg_list_scalar_arg_dtypes(self.parsed_args)
-                + [self.index_dtype, self.index_dtype, None, None])
+        update_scalar_arg_dtypes = [
+                *get_arg_list_scalar_arg_dtypes(self.parsed_args),
+                self.index_dtype, self.index_dtype, None, None]
+
         if self.is_segmented:
             # g_first_segment_start_in_interval
             update_scalar_arg_dtypes.append(None)
@@ -1561,7 +1569,8 @@ class GenericScanKernel(GenericScanKernelBase):
 
         # {{{ first level scan of interval (one interval per block)
 
-        scan1_args = data_args + [
+        scan1_args = [
+                *data_args,
                 partial_scan_buffer.data, n, interval_size, interval_results.data,
                 ]
 
@@ -1585,12 +1594,15 @@ class GenericScanKernel(GenericScanKernelBase):
         # can scan at most one interval
         assert interval_size >= num_intervals
 
-        scan2_args = data_args + [
+        scan2_args = [
+                *data_args,
                 interval_results.data,  # interval_sums
                 ]
+
         if self.is_segmented:
             scan2_args.append(first_segment_start_in_interval.data)
-        scan2_args = scan2_args + [
+        scan2_args = [
+                *scan2_args,
                 interval_results.data,  # partial_scan_buffer
                 num_intervals, interval_size]
 
@@ -1602,7 +1614,8 @@ class GenericScanKernel(GenericScanKernelBase):
 
         # {{{ update intervals with result of interval scan
 
-        upd_args = data_args + [
+        upd_args = [
+                *data_args,
                 n, interval_size, interval_results.data, partial_scan_buffer.data]
         if self.is_segmented:
             upd_args.append(first_segment_start_in_interval.data)
@@ -1708,10 +1721,11 @@ class GenericDebugScanKernel(GenericScanKernelBase):
 
         scan_prg = cl.Program(self.context, scan_src).build(self.options)
         self.kernel = getattr(scan_prg, f"{self.name_prefix}_debug_scan")
-        scalar_arg_dtypes = (
-                [None]
-                + get_arg_list_scalar_arg_dtypes(self.parsed_args)
-                + [self.index_dtype])
+        scalar_arg_dtypes = [
+                None,
+                *get_arg_list_scalar_arg_dtypes(self.parsed_args),
+                self.index_dtype,
+                ]
         self.kernel.set_scalar_arg_dtypes(scalar_arg_dtypes)
 
     def __call__(self, *args: Any, **kwargs: Any) -> cl.Event:
@@ -1758,8 +1772,7 @@ class GenericDebugScanKernel(GenericScanKernelBase):
 
         # }}}
 
-        return self.kernel(queue, (1,), (1,),
-                *(data_args + [n]), wait_for=wait_for)
+        return self.kernel(queue, (1,), (1,), *([*data_args, n]), wait_for=wait_for)
 
 # }}}
 
