@@ -126,27 +126,30 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
+import re
 from abc import ABC, abstractmethod
 from sys import intern
 from typing import Any, List, Optional, Union
 
-# Do not add a pyopencl import here: This will add an import cycle.
-
 import numpy as np
+
 from pytools import memoize, memoize_method
-from pyopencl._cl import bitlog2, get_cl_header_version  # noqa: F401
 from pytools.persistent_dict import KeyBuilder as KeyBuilderBase
 
-import re
-
+from pyopencl._cl import bitlog2, get_cl_header_version  # noqa: F401
 from pyopencl.compyte.dtypes import (  # noqa: F401
-        get_or_register_dtype, TypeNameNotKnown,
-        register_dtype, dtype_to_ctype)
+    TypeNameNotKnown,
+    dtype_to_ctype,
+    get_or_register_dtype,
+    register_dtype,
+)
+
+
+# Do not add a pyopencl import here: This will add an import cycle.
 
 
 def _register_types():
-    from pyopencl.compyte.dtypes import (
-            TYPE_REGISTRY, fill_registry_with_opencl_c_types)
+    from pyopencl.compyte.dtypes import TYPE_REGISTRY, fill_registry_with_opencl_c_types
 
     fill_registry_with_opencl_c_types(TYPE_REGISTRY)
 
@@ -159,18 +162,17 @@ _register_types()
 
 # {{{ imported names
 
-from pyopencl._cl import (  # noqa: F401
-        PooledBuffer, AllocatorBase, DeferredAllocator,
-        ImmediateAllocator, MemoryPool,
-        )
+from pyopencl._cl import (
+    AllocatorBase,
+    DeferredAllocator,
+    ImmediateAllocator,
+    MemoryPool,
+    PooledBuffer,
+)
 
 
 if get_cl_header_version() >= (2, 0):
-    from pyopencl._cl import (  # noqa: F401
-            SVMPool,
-            PooledSVM,
-            SVMAllocator,
-            )
+    from pyopencl._cl import PooledSVM, SVMAllocator, SVMPool
 
 # }}}
 
@@ -354,96 +356,98 @@ def _monkeypatch_svm_docstrings():
 
     # {{{ PooledSVM
 
-    PooledSVM.__doc__ = """
-    An object representing a :class:`SVMPool`-based allocation of
-    :ref:`svm`.  Analogous to :class:`~pyopencl.SVMAllocation`, however once
-    this object is deleted, its associated device memory is returned to the
-    pool from which it came.
+    PooledSVM.__doc__ = (  # pylint: disable=possibly-used-before-assignment
+        """An object representing a :class:`SVMPool`-based allocation of
+        :ref:`svm`.  Analogous to :class:`~pyopencl.SVMAllocation`, however once
+        this object is deleted, its associated device memory is returned to the
+        pool from which it came.
 
-    .. versionadded:: 2022.2
+        .. versionadded:: 2022.2
 
-    .. note::
+        .. note::
 
-        If the :class:`SVMAllocator` for the :class:`SVMPool` that allocated an
-        object of this type is associated with an (in-order)
-        :class:`~pyopencl.CommandQueue`, sufficient synchronization is provided
-        to ensure operations enqueued before deallocation complete before
-        operations from a different use (possibly in a different queue) are
-        permitted to start. This applies when :class:`release` is called and
-        also when the object is freed automatically by the garbage collector.
+            If the :class:`SVMAllocator` for the :class:`SVMPool` that allocated an
+            object of this type is associated with an (in-order)
+            :class:`~pyopencl.CommandQueue`, sufficient synchronization is provided
+            to ensure operations enqueued before deallocation complete before
+            operations from a different use (possibly in a different queue) are
+            permitted to start. This applies when :class:`release` is called and
+            also when the object is freed automatically by the garbage collector.
 
-    Is a :class:`pyopencl.SVMPointer`.
+        Is a :class:`pyopencl.SVMPointer`.
 
-    Supports structural equality and hashing.
+        Supports structural equality and hashing.
 
-    .. automethod:: release
+        .. automethod:: release
 
-        Return the held memory to the pool. See the note about synchronization
-        behavior during deallocation above.
+            Return the held memory to the pool. See the note about synchronization
+            behavior during deallocation above.
 
-    .. automethod:: enqueue_release
+        .. automethod:: enqueue_release
 
-        Synonymous to :meth:`release`, for consistency with
-        :class:`~pyopencl.SVMAllocation`. Note that, unlike
-        :meth:`pyopencl.SVMAllocation.enqueue_release`, specifying a queue
-        or events to be waited for is not supported.
+            Synonymous to :meth:`release`, for consistency with
+            :class:`~pyopencl.SVMAllocation`. Note that, unlike
+            :meth:`pyopencl.SVMAllocation.enqueue_release`, specifying a queue
+            or events to be waited for is not supported.
 
-    .. automethod:: bind_to_queue
+        .. automethod:: bind_to_queue
 
-        Analogous to :meth:`pyopencl.SVMAllocation.bind_to_queue`.
+            Analogous to :meth:`pyopencl.SVMAllocation.bind_to_queue`.
 
-    .. automethod:: unbind_from_queue
+        .. automethod:: unbind_from_queue
 
-        Analogous to :meth:`pyopencl.SVMAllocation.unbind_from_queue`.
-    """
+            Analogous to :meth:`pyopencl.SVMAllocation.unbind_from_queue`.
+        """)
 
     # }}}
 
     # {{{ SVMAllocator
 
-    SVMAllocator.__doc__ = """
-    .. versionadded:: 2022.2
+    SVMAllocator.__doc__ = (  # pylint: disable=possibly-used-before-assignment
+        """
+        .. versionadded:: 2022.2
 
-    .. automethod:: __init__
+        .. automethod:: __init__
 
-        :arg flags: See :class:`~pyopencl.svm_mem_flags`.
-        :arg queue: If not specified, allocations will be freed
-            eagerly, irrespective of whether pending/enqueued operations
-            are still using the memory.
+            :arg flags: See :class:`~pyopencl.svm_mem_flags`.
+            :arg queue: If not specified, allocations will be freed
+                eagerly, irrespective of whether pending/enqueued operations
+                are still using the memory.
 
-            If specified, deallocation of memory will be enqueued
-            with the given queue, and will only be performed
-            after previously-enqueue operations in the queue have
-            completed.
+                If specified, deallocation of memory will be enqueued
+                with the given queue, and will only be performed
+                after previously-enqueue operations in the queue have
+                completed.
 
-            It is an error to specify an out-of-order queue.
+                It is an error to specify an out-of-order queue.
 
-            .. warning::
+                .. warning::
 
-                Not specifying a queue will typically lead to undesired
-                behavior, including crashes and memory corruption.
-                See the warning in :ref:`svm`.
+                    Not specifying a queue will typically lead to undesired
+                    behavior, including crashes and memory corruption.
+                    See the warning in :ref:`svm`.
 
-    .. automethod:: __call__
+        .. automethod:: __call__
 
-        Return a :class:`~pyopencl.SVMAllocation` of the given *size*.
-    """
+            Return a :class:`~pyopencl.SVMAllocation` of the given *size*.
+        """)
 
     # }}}
 
     # {{{ SVMPool
 
-    SVMPool.__doc__ = remove_common_indentation("""
-    A memory pool for OpenCL device memory in :ref:`SVM <svm>` form.
-    *allocator* must be an instance of :class:`SVMAllocator`.
+    SVMPool.__doc__ = (  # pylint: disable=possibly-used-before-assignment
+        remove_common_indentation("""
+        A memory pool for OpenCL device memory in :ref:`SVM <svm>` form.
+        *allocator* must be an instance of :class:`SVMAllocator`.
 
-    .. versionadded:: 2022.2
+        .. versionadded:: 2022.2
 
-    .. automethod:: __init__
-    .. automethod:: __call__
+        .. automethod:: __init__
+        .. automethod:: __call__
 
-        Return a :class:`PooledSVM` of the given *size*.
-    """) + _MEMPOOL_IFACE_DOCS
+            Return a :class:`PooledSVM` of the given *size*.
+        """) + _MEMPOOL_IFACE_DOCS)
 
     # }}}
 
@@ -518,13 +522,14 @@ def first_arg_dependent_memoize_nested(nested_func):
                 nested_func.__code__.co_firstlineno))
 
     from inspect import currentframe
+
     # prevent ref cycle
     try:
         caller_frame = currentframe().f_back
         cache_context = caller_frame.f_globals[
                 caller_frame.f_code.co_name]
     finally:
-        #del caller_frame
+        # del caller_frame
         pass
 
     try:
@@ -560,6 +565,8 @@ def clear_first_arg_caches():
 
 
 import atexit
+
+
 atexit.register(clear_first_arg_caches)
 
 # }}}
@@ -654,7 +661,8 @@ def get_pyopencl_fixture_arg_names(metafunc, extra_arg_names=None):
     supported_arg_names = [
             "platform", "device",
             "ctx_factory", "ctx_getter",
-            ] + extra_arg_names
+            *extra_arg_names
+            ]
 
     arg_names = []
     for arg in supported_arg_names:
@@ -979,7 +987,7 @@ class _CDeclList:
             return
 
         for _name, field_data in sorted(dtype.fields.items()):
-            field_dtype, offset = field_data[:2]
+            field_dtype, _offset = field_data[:2]
             self.add_dtype(field_dtype)
 
         _, cdecl = match_dtype_to_c_struct(
@@ -1063,7 +1071,7 @@ def match_dtype_to_c_struct(device, name, dtype, context=None):
 
     c_fields = []
     for field_name, dtype_and_offset in fields:
-        field_dtype, offset = dtype_and_offset[:2]
+        field_dtype, _offset = dtype_and_offset[:2]
         if hasattr(field_dtype, "subdtype") and field_dtype.subdtype is not None:
             array_dtype = field_dtype.subdtype[0]
             if hasattr(array_dtype, "subdtype") and array_dtype.subdtype is not None:
@@ -1088,7 +1096,7 @@ def match_dtype_to_c_struct(device, name, dtype, context=None):
 
     cdl = _CDeclList(device)
     for _field_name, dtype_and_offset in fields:
-        field_dtype, offset = dtype_and_offset[:2]
+        field_dtype, _offset = dtype_and_offset[:2]
         cdl.add_dtype(field_dtype)
 
     pre_decls = cdl.get_declarations()
@@ -1122,7 +1130,8 @@ def match_dtype_to_c_struct(device, name, dtype, context=None):
     prg = cl.Program(context, src)
     knl = prg.build(devices=[device]).get_size_and_offsets
 
-    import pyopencl.array  # noqa: F401
+    import pyopencl.array
+
     result_buf = cl.array.empty(queue, 1+len(fields), np.uint32)
     knl(queue, (1,), (1,), result_buf.data)
     queue.finish()
@@ -1366,6 +1375,8 @@ class _TemplateRenderer:
                 parsed_arg = arg
             elif isinstance(arg, tuple):
                 parsed_arg = ScalarArg(self.parse_type(arg[0]), arg[1])
+            else:
+                raise TypeError("unexpected argument type: %s" % type(arg))
 
             parsed_args.append(parsed_arg)
 
