@@ -27,6 +27,7 @@
 #define PY_ARRAY_UNIQUE_SYMBOL pyopencl_ARRAY_API
 
 #include "wrap_cl.hpp"
+#include <nanobind/intrusive/counter.inl>
 
 
 
@@ -47,15 +48,30 @@ static bool import_numpy_helper()
   return true;
 }
 
-PYBIND11_MODULE(_cl, m)
+NB_MODULE(_cl, m)
 {
+  py::intrusive_init(
+    [](PyObject *o) noexcept {
+        py::gil_scoped_acquire guard;
+        Py_INCREF(o);
+    },
+    [](PyObject *o) noexcept {
+        py::gil_scoped_acquire guard;
+        Py_DECREF(o);
+    });
+
   if (!import_numpy_helper())
-    throw py::error_already_set();
+    throw py::python_error();
 
   pyopencl_expose_constants(m);
   pyopencl_expose_part_1(m);
   pyopencl_expose_part_2(m);
   pyopencl_expose_mempool(m);
+
+#ifdef NDEBUG
+  // See https://github.com/inducer/pyopencl/issues/758 for context.
+  py::set_leak_warnings(false);
+#endif
 }
 
 // vim: foldmethod=marker
