@@ -65,22 +65,6 @@ namespace pyopencl {
   }
 
 #endif
-
-#if PYOPENCL_CL_VERSION >= 0x2000
-  class svm_pointer_as_buffer
-  {
-    private:
-      svm_pointer &m_ptr;
-
-    public:
-      svm_pointer_as_buffer(svm_pointer &ptr)
-        : m_ptr(ptr)
-      { }
-
-      svm_pointer &ptr() const
-      { return m_ptr; }
-  };
-#endif
 }
 
 
@@ -89,6 +73,19 @@ namespace pyopencl {
 using namespace pyopencl;
 
 
+static PyCFunctionWithKeywords dummy_init = [](PyObject *, PyObject *,
+                                        PyObject *) -> PyObject * {
+    PyErr_SetString(PyExc_RuntimeError, "This should never be called!");
+    return nullptr;
+};
+
+static PyType_Slot init_slots[] {
+    // the presence of this slot enables normal object construction via __init__ and __new__
+    // instead of an optimized codepath within nanobind that skips these. That in turn
+    // makes it possible to intercept calls and implement custom logic.
+    { Py_tp_init, (void *) dummy_init },
+    { 0, nullptr }
+};
 
 
 void pyopencl_expose_part_2(py::module_ &m)
@@ -100,32 +97,41 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef cl_image_desc cls;
     py::class_<cls>(m, "ImageDescriptor")
       .def(py::init<>())
-      .def_readwrite("image_type", &cls::image_type)
-      .def_property("shape", &image_desc_dummy_getter, image_desc_set_shape)
-      .def_readwrite("array_size", &cls::image_array_size)
-      .def_property("pitches", &image_desc_dummy_getter, image_desc_set_pitches)
-      .def_readwrite("num_mip_levels", &cls::num_mip_levels)
-      .def_readwrite("num_samples", &cls::num_samples)
-      .def_property("buffer", &image_desc_dummy_getter, image_desc_set_buffer)
+      .def_rw("image_type", &cls::image_type)
+      .def_prop_rw("shape", &image_desc_dummy_getter, image_desc_set_shape)
+      .def_rw("array_size", &cls::image_array_size)
+      .def_prop_rw("pitches", &image_desc_dummy_getter, image_desc_set_pitches)
+      .def_rw("num_mip_levels", &cls::num_mip_levels)
+      .def_rw("num_samples", &cls::num_samples)
+      .def_prop_rw("buffer", &image_desc_dummy_getter, image_desc_set_buffer,
+                   py::arg("buffer").none()
+                 )
       ;
   }
 #endif
 
   {
     typedef image cls;
-    py::class_<cls, memory_object>(m, "Image", py::dynamic_attr())
-      .def(
-          py::init(
-            [](
-              context const &ctx,
-              cl_mem_flags flags,
-              cl_image_format const &fmt,
-              py::sequence shape,
-              py::sequence pitches,
-              py::object buffer)
-            {
-              return create_image(ctx, flags, fmt, shape, pitches, buffer);
-            }),
+    // https://github.com/wjakob/nanobind/issues/750
+    py::class_<cls, memory_object>(m, "Image", py::dynamic_attr(), py::type_slots(init_slots))
+      .def_static(
+          "_custom_init",
+          [](
+            py::handle_t<cls> h,
+            context const &ctx,
+            cl_mem_flags flags,
+            cl_image_format const &fmt,
+            py::sequence shape,
+            py::sequence pitches,
+            py::object buffer)
+          {
+            if (py::inst_ready(h))
+              py::raise_type_error("Image is already initialized!");
+            image *self = py::inst_ptr<cls>(h);
+            create_image(self, ctx, flags, fmt, shape, pitches, buffer);
+            py::inst_mark_ready(h);
+          },
+          py::arg("h"),
           py::arg("context"),
           py::arg("flags"),
           py::arg("format"),
@@ -134,17 +140,23 @@ void pyopencl_expose_part_2(py::module_ &m)
           py::arg("hostbuf")=py::none()
           )
 #if PYOPENCL_CL_VERSION >= 0x1020
-      .def(
-          py::init(
-            [](
-              context const &ctx,
-              cl_mem_flags flags,
-              cl_image_format const &fmt,
-              cl_image_desc &desc,
-              py::object buffer)
-            {
-              return create_image_from_desc(ctx, flags, fmt, desc, buffer);
-            }),
+      .def_static(
+          "_custom_init",
+          [](
+            py::handle_t<cls> h,
+            context const &ctx,
+            cl_mem_flags flags,
+            cl_image_format const &fmt,
+            cl_image_desc &desc,
+            py::object buffer)
+          {
+            if (py::inst_ready(h))
+              py::raise_type_error("Image is already initialized!");
+            image *self = py::inst_ptr<cls>(h);
+            create_image_from_desc(self, ctx, flags, fmt, desc, buffer);
+            py::inst_mark_ready(h);
+          },
+          py::arg("h"),
           py::arg("context"),
           py::arg("flags"),
           py::arg("format"),
@@ -160,16 +172,16 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef cl_image_format cls;
     py::class_<cls>(m, "ImageFormat")
       .def(
-          py::init(
-            [](cl_channel_order ord, cl_channel_type tp)
-            {
-              return make_image_format(ord, tp);
-            }))
-      .def_readwrite("channel_order", &cls::image_channel_order)
-      .def_readwrite("channel_data_type", &cls::image_channel_data_type)
-      .def_property_readonly("channel_count", &get_image_format_channel_count)
-      .def_property_readonly("dtype_size", &get_image_format_channel_dtype_size)
-      .def_property_readonly("itemsize", &get_image_format_item_size)
+          "__init__",
+          [](cls *self, cl_channel_order ord, cl_channel_type tp)
+          {
+            set_image_format(self, ord, tp);
+          })
+      .def_rw("channel_order", &cls::image_channel_order)
+      .def_rw("channel_data_type", &cls::image_channel_data_type)
+      .def_prop_ro("channel_count", &get_image_format_channel_count)
+      .def_prop_ro("dtype_size", &get_image_format_channel_dtype_size)
+      .def_prop_ro("itemsize", &get_image_format_item_size)
       ;
   }
 
@@ -246,16 +258,17 @@ void pyopencl_expose_part_2(py::module_ &m)
     py::class_<cls, memory_object>(m, "Pipe", py::dynamic_attr())
 #if PYOPENCL_CL_VERSION >= 0x2000
       .def(
-          py::init(
-            [](
-              context const &ctx,
-              cl_mem_flags flags,
-              cl_uint pipe_packet_size,
-              cl_uint pipe_max_packets,
-              py::sequence py_props)
-            {
-              return create_pipe(ctx, flags, pipe_packet_size, pipe_max_packets, py_props);
-            }),
+          "__init__",
+          [](
+            cls *self,
+            context const &ctx,
+            cl_mem_flags flags,
+            cl_uint pipe_packet_size,
+            cl_uint pipe_max_packets,
+            py::sequence py_props)
+          {
+            create_pipe(self, ctx, flags, pipe_packet_size, pipe_max_packets, py_props);
+          },
           py::arg("context"),
           py::arg("flags"),
           py::arg("packet_size"),
@@ -318,9 +331,9 @@ void pyopencl_expose_part_2(py::module_ &m)
       // For consistency, it may seem appropriate to use int_ptr here, but
       // that would work on both buffers and SVM, and passing a buffer pointer to
       // a kernel is going to lead to a bad time.
-      .def_property_readonly("svm_ptr",
+      .def_prop_ro("svm_ptr",
           [](cls &self) { return (intptr_t) self.svm_ptr(); })
-      .def_property_readonly("size", [](cls &self) -> py::object
+      .def_prop_ro("size", [](cls &self) -> py::object
           {
             try
             {
@@ -331,43 +344,25 @@ void pyopencl_expose_part_2(py::module_ &m)
               return py::none();
             }
           })
-      .def_property_readonly("buf", [](cls &self) -> svm_pointer_as_buffer * {
-            return new svm_pointer_as_buffer(self);
-          }, py::return_value_policy::reference_internal)
-      ;
-  }
-
-  {
-    typedef svm_pointer_as_buffer cls;
-    py::class_<cls>(m, "_SVMPointerAsBuffer", pybind11::buffer_protocol())
-      .def_buffer([](cls &self) -> pybind11::buffer_info
-          {
+      .def_prop_ro("buf", [](cls &self) -> py::ndarray<py::numpy, unsigned char, py::ndim<1>> {
             size_t size;
             try
             {
-              size = self.ptr().size();
+              size = self.size();
             }
             catch (size_not_available)
             {
               throw pyopencl::error("SVMPointer buffer protocol", CL_INVALID_VALUE,
                   "size of SVM is not known");
             }
-            return pybind11::buffer_info(
-                // Pointer to buffer
-                self.ptr().svm_ptr(),
-                // Size of one scalar
-                sizeof(unsigned char),
-                // Python struct-style format descriptor
-                pybind11::format_descriptor<unsigned char>::format(),
-                // Number of dimensions
-                1,
-                // Buffer dimensions
-                { size },
-                // Strides (in bytes) for each index
-                { sizeof(unsigned char) }
-                );
-          })
-    ;
+
+            return py::ndarray<py::numpy, unsigned char, py::ndim<1>>(
+              /* data = */ self.svm_ptr(),
+              /* ndim = */ 1,
+              /* shape pointer = */ &size,
+              /* owner = */ py::handle());
+          }, py::rv_policy::reference_internal)
+      ;
   }
 
   // }}}
@@ -378,6 +373,7 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef svm_arg_wrapper cls;
     py::class_<cls, svm_pointer>(m, "SVM", py::dynamic_attr())
       .def(py::init<py::object>())
+      .def_prop_ro("mem", &cls::mem)
       ;
   }
 
@@ -388,7 +384,7 @@ void pyopencl_expose_part_2(py::module_ &m)
   {
     typedef svm_allocation cls;
     py::class_<cls, svm_pointer>(m, "SVMAllocation", py::dynamic_attr())
-      .def(py::init<std::shared_ptr<context>, size_t, cl_uint, cl_svm_mem_flags, const command_queue *>(),
+      .def(py::init<py::ref<context>, size_t, cl_uint, cl_svm_mem_flags, const command_queue *>(),
           py::arg("context"),
           py::arg("size"),
           py::arg("alignment"),
@@ -402,15 +398,14 @@ void pyopencl_expose_part_2(py::module_ &m)
           py::arg("queue").none(true)=py::none(),
           py::arg("wait_for").none(true)=py::none()
           )
-      .def(py::self == py::self)
-      .def(py::self != py::self)
+      PYOPENCL_EXPOSE_EQUALITY_TESTS
       .def("__hash__", [](cls &self) { return (intptr_t) self.svm_ptr(); })
       .def("bind_to_queue", &cls::bind_to_queue,
           py::arg("queue"))
       .DEF_SIMPLE_METHOD(unbind_from_queue)
 
       // only for diagnostic/debugging/testing purposes!
-      .def_property_readonly("_queue",
+      .def_prop_ro("_queue",
           [](cls const &self) -> py::object
           {
             cl_command_queue queue = self.queue();
@@ -479,8 +474,7 @@ void pyopencl_expose_part_2(py::module_ &m)
 #endif
       .def(py::init<context const &, bool, cl_addressing_mode, cl_filter_mode>())
       .DEF_SIMPLE_METHOD(get_info)
-      .def(py::self == py::self)
-      .def(py::self != py::self)
+      PYOPENCL_EXPOSE_EQUALITY_TESTS
       .def("__hash__", &cls::hash)
       PYOPENCL_EXPOSE_TO_FROM_INT_PTR(cl_sampler)
       ;
@@ -500,19 +494,19 @@ void pyopencl_expose_part_2(py::module_ &m)
 
     py::class_<cls>(m, "_Program", py::dynamic_attr())
       .def(
-          py::init(
-            [](context &ctx, std::string const &src)
-            {
-              return create_program_with_source(ctx, src);
-            }),
+          "__init__",
+          [](cls *self, context &ctx, std::string const &src)
+          {
+            create_program_with_source(self, ctx, src);
+          },
           py::arg("context"),
           py::arg("src"))
       .def(
-          py::init(
-            [](context &ctx, py::sequence devices, py::sequence binaries)
-            {
-              return create_program_with_binary(ctx, devices, binaries);
-            }),
+          "__init__",
+          [](cls *self, context &ctx, py::sequence devices, py::sequence binaries)
+          {
+            return create_program_with_binary(self, ctx, devices, binaries);
+          },
           py::arg("context"),
           py::arg("devices"),
           py::arg("binaries"))
@@ -547,8 +541,7 @@ void pyopencl_expose_part_2(py::module_ &m)
           py::arg("spec_id"),
           py::arg("buffer"))
 #endif
-      .def(py::self == py::self)
-      .def(py::self != py::self)
+      PYOPENCL_EXPOSE_EQUALITY_TESTS
       .def("__hash__", &cls::hash)
       .def("all_kernels", create_kernels_in_program)
       PYOPENCL_EXPOSE_TO_FROM_INT_PTR(cl_program)
@@ -570,7 +563,8 @@ void pyopencl_expose_part_2(py::module_ &m)
   {
     typedef kernel cls;
     py::class_<cls>(m, "Kernel", py::dynamic_attr())
-      .def(py::init<const program &, std::string const &>())
+      .def(py::init<py::object, std::string const &>())
+      .def_prop_ro("_source", &cls::source)
       .DEF_SIMPLE_METHOD(get_info)
       .DEF_SIMPLE_METHOD(get_work_group_info)
 #if PYOPENCL_CL_VERSION >= 0x2010
@@ -607,8 +601,7 @@ void pyopencl_expose_part_2(py::module_ &m)
 #if PYOPENCL_CL_VERSION >= 0x1020
       .DEF_SIMPLE_METHOD(get_arg_info)
 #endif
-      .def(py::self == py::self)
-      .def(py::self != py::self)
+      PYOPENCL_EXPOSE_EQUALITY_TESTS
       .def("__hash__", &cls::hash)
       PYOPENCL_EXPOSE_TO_FROM_INT_PTR(cl_kernel)
 #if PYOPENCL_CL_VERSION >= 0x2010
@@ -618,6 +611,9 @@ void pyopencl_expose_part_2(py::module_ &m)
           py::arg("input_value").none(true)=py::none()
           )
 #endif
+      .def("__call__", &cls::enqueue)
+      .def("set_args", &cls::set_args)
+      .def("_set_enqueue_and_set_args", &cls::set_enqueue_and_set_args)
       ;
   }
 
@@ -627,7 +623,7 @@ void pyopencl_expose_part_2(py::module_ &m)
       .def(
           py::init<size_t>(),
           py::arg("size"))
-      .def_property_readonly("size", &cls::size)
+      .def_prop_ro("size", &cls::size)
       ;
   }
 
@@ -658,11 +654,11 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef gl_buffer cls;
     py::class_<cls, memory_object>(m, "GLBuffer", py::dynamic_attr())
       .def(
-          py::init(
-            [](context &ctx, cl_mem_flags flags, GLuint bufobj)
-            {
-              return create_from_gl_buffer(ctx, flags, bufobj);
-            }),
+          "__init__",
+          [](cls *self, context &ctx, cl_mem_flags flags, GLuint bufobj)
+          {
+            create_from_gl_buffer(self, ctx, flags, bufobj);
+          },
           py::arg("context"),
           py::arg("flags"),
           py::arg("bufobj"))
@@ -674,11 +670,11 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef gl_renderbuffer cls;
     py::class_<cls, memory_object>(m, "GLRenderBuffer", py::dynamic_attr())
       .def(
-          py::init(
-            [](context &ctx, cl_mem_flags flags, GLuint bufobj)
-            {
-              return create_from_gl_renderbuffer(ctx, flags, bufobj);
-            }),
+          "__init__",
+          [](cls *self, context &ctx, cl_mem_flags flags, GLuint bufobj)
+          {
+            create_from_gl_renderbuffer(self, ctx, flags, bufobj);
+          },
           py::arg("context"),
           py::arg("flags"),
           py::arg("bufobj"))
@@ -690,12 +686,12 @@ void pyopencl_expose_part_2(py::module_ &m)
     typedef gl_texture cls;
     py::class_<cls, image>(m, "GLTexture", py::dynamic_attr())
       .def(
-          py::init(
-            [](context &ctx, cl_mem_flags flags, GLenum texture_target,
-              GLint miplevel, GLuint texture, unsigned dims)
-            {
-              return create_from_gl_texture(ctx, flags, texture_target, miplevel, texture, dims);
-            }),
+          "__init__",
+          [](cls *self, context &ctx, cl_mem_flags flags, GLenum texture_target,
+            GLint miplevel, GLuint texture, unsigned dims)
+          {
+            create_from_gl_texture(self, ctx, flags, texture_target, miplevel, texture, dims);
+          },
           py::arg("context"),
           py::arg("flags"),
           py::arg("texture_target"),
