@@ -28,16 +28,21 @@
 #define PYCUDA_WRAP_HELPERS_HEADER_SEEN
 
 
-#include <pybind11/pybind11.h>
-#include <pybind11/operators.h>
+#include <nanobind/nanobind.h>
+#include <nanobind/stl/string.h>
+#include <nanobind/intrusive/counter.h>
+#include <nanobind/intrusive/ref.h>
+#include <nanobind/ndarray.h>
 
 
-namespace py = pybind11;
+namespace py = nanobind;
 
 
 #define ENUM_VALUE(NAME) \
   value(#NAME, NAME)
 
+// {{{ DEF_SIMPLE_XXX
+
 #define DEF_SIMPLE_METHOD(NAME) \
   def(#NAME, &cls::NAME)
 
@@ -120,7 +125,7 @@ namespace py = pybind11;
 #define PYOPENCL_PARSE_NUMPY_ARRAY_SPEC \
     PyArray_Descr *tp_descr; \
     if (PyArray_DescrConverter(dtype.ptr(), &tp_descr) != NPY_SUCCEED) \
-      throw py::error_already_set(); \
+      throw py::python_error(); \
     \
     std::vector<npy_intp> shape; \
     try \
@@ -132,13 +137,13 @@ namespace py = pybind11;
       COPY_PY_LIST(npy_intp, shape); \
     } \
     \
-    NPY_ORDER order = PyArray_CORDER; \
+    NPY_ORDER order = NPY_CORDER; \
     PyArray_OrderConverter(py_order.ptr(), &order); \
     \
     int ary_flags = 0; \
-    if (order == PyArray_FORTRANORDER) \
+    if (order == NPY_FORTRANORDER) \
       ary_flags |= NPY_FARRAY; \
-    else if (order == PyArray_CORDER) \
+    else if (order == NPY_CORDER) \
       ary_flags |= NPY_CARRAY; \
     else \
       throw std::runtime_error("unrecognized order specifier"); \
@@ -162,7 +167,7 @@ namespace
   template <typename T>
   inline py::object handle_from_new_ptr(T *ptr)
   {
-    return py::cast(ptr, py::return_value_policy::take_ownership);
+    return py::cast(ptr, py::rv_policy::take_ownership);
   }
 
   template <typename T, typename ClType>
@@ -192,10 +197,18 @@ namespace
       ":mod:`pyopencl`." \
       "\n\n.. versionadded:: 2013.2\n" \
       "\n\n.. versionchanged:: 2016.1\n\n    *retain* added.") \
-  .def_property_readonly("int_ptr", to_int_ptr<cls>, \
+  .def_prop_ro("int_ptr", to_int_ptr<cls>, \
       "Return an integer corresponding to the pointer value " \
       "of the underlying :c:type:`" #CL_TYPENAME "`. " \
       "Use :meth:`from_int_ptr` to turn back into a Python object." \
       "\n\n.. versionadded:: 2013.2\n") \
 
+#define PYOPENCL_EXPOSE_EQUALITY_TESTS \
+    /* this relies on nanobind overload resolution going in order of registration */ \
+    .def("__eq__", [](cls const &self, cls const &other) { return self == other; }) \
+    .def("__eq__", [](cls const &self, py::object obj) { return false; }, py::arg("obj").none())
+
+
 #endif
+
+// vim: foldmethod=marker
