@@ -20,22 +20,31 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
+import sys
+
 import numpy as np
 import numpy.linalg as la
-import sys
-from pytools import memoize
+import pytest
 from test_array import general_clrand
 
-import pytest
+from pytools import memoize
 
 import pyopencl as cl
 import pyopencl.array
-from pyopencl.tools import (  # noqa: F401
-        pytest_generate_tests_for_pyopencl as pytest_generate_tests)
-from pyopencl.characterize import has_double_support, has_struct_arg_count_bug
-from pyopencl.scan import (InclusiveScanKernel, ExclusiveScanKernel,
-        GenericScanKernel, GenericDebugScanKernel)
-from pyopencl.characterize import get_pocl_version
+from pyopencl.characterize import (
+    get_pocl_version,
+    has_double_support,
+    has_struct_arg_count_bug,
+)
+from pyopencl.scan import (
+    ExclusiveScanKernel,
+    GenericDebugScanKernel,
+    GenericScanKernel,
+    InclusiveScanKernel,
+)
+from pyopencl.tools import (
+    pytest_generate_tests_for_pyopencl as pytest_generate_tests,  # noqa: F401
+)
 
 
 # {{{ elementwise
@@ -486,8 +495,8 @@ def test_struct_reduce(ctx_factory):
             arguments="__global int *x", preamble=preamble)
 
     minmax = red(a_gpu).get()
-    #print minmax["cur_min"], minmax["cur_max"]
-    #print np.min(a), np.max(a)
+    # print(minmax["cur_min"], minmax["cur_max"])
+    # print(np.min(a), np.max(a))
 
     assert abs(minmax["cur_min"] - np.min(a)) < 1e-5
     assert abs(minmax["cur_max"] - np.max(a)) < 1e-5
@@ -642,7 +651,7 @@ def test_copy_if(ctx_factory):
 
         crit = a_dev.dtype.type(300)
         selected = a[a > crit]
-        selected_dev, count_dev, evt = copy_if(
+        selected_dev, count_dev, _evt = copy_if(
                 a_dev, "ary[i] > myval", [("myval", crit)])
 
         assert (selected_dev.get()[:count_dev.get()] == selected).all()
@@ -669,7 +678,7 @@ def test_partition(ctx_factory):
         false_host = a[a <= crit]
 
         from pyopencl.algorithm import partition
-        true_dev, false_dev, count_true_dev, evt = partition(
+        true_dev, false_dev, count_true_dev, _evt = partition(
                 a_dev, "ary[i] > myval", [("myval", crit)])
 
         count_true_dev = count_true_dev.get()
@@ -695,7 +704,7 @@ def test_unique(ctx_factory):
         a_unique_host = np.unique(a)
 
         from pyopencl.algorithm import unique
-        a_unique_dev, count_unique_dev, evt = unique(a_dev)
+        a_unique_dev, count_unique_dev, _evt = unique(a_dev)
 
         count_unique_dev = count_unique_dev.get()
 
@@ -747,7 +756,7 @@ def test_segmented_scan(ctx_factory):
     dtype = np.int32
     ctype = dtype_to_ctype(dtype)
 
-    #for is_exclusive in [False, True]:
+    # for is_exclusive in [False, True]:
     for is_exclusive in [True, False]:
         if is_exclusive:
             output_statement = "out[i] = prev_item"
@@ -765,6 +774,7 @@ def test_segmented_scan(ctx_factory):
 
         np.set_printoptions(threshold=2000)
         from random import randrange
+
         from pyopencl.clrandom import rand as clrand
         for n in scan_test_counts:
             a_dev = clrand(queue, (n,), dtype=dtype, a=0, b=10)
@@ -788,8 +798,8 @@ def test_segmented_scan(ctx_factory):
                     seg_boundaries_values.append(seg_boundaries)
 
             for seg_boundaries in seg_boundaries_values:
-                #print "BOUNDARIES", seg_boundaries
-                #print a
+                # print("BOUNDARIES", seg_boundaries)
+                # print(a)
 
                 seg_boundary_flags = np.zeros(n, dtype=np.uint8)
                 seg_boundary_flags[seg_boundaries] = 1
@@ -813,12 +823,12 @@ def test_segmented_scan(ctx_factory):
                         result_host[seg_start:seg_end] = np.cumsum(
                                 a[seg_start:seg_end])
 
-                #print "REF", result_host
+                # print("REF", result_host)
 
                 result_dev = cl.array.empty_like(a_dev)
                 knl(a_dev, seg_boundary_flags_dev, result_dev)
 
-                #print "RES", result_dev
+                # print("RES", result_dev)
                 is_correct = (result_dev.get() == result_host).all()
                 if not is_correct:
                     diff = result_dev.get() - result_host
@@ -847,8 +857,8 @@ def test_sort(ctx_factory, scan_kernel):
     sort = RadixSort(context, "int *ary", key_expr="ary[i]",
             sort_arg_names=["ary"], scan_kernel=scan_kernel)
 
-    from pyopencl.clrandom import RanluxGenerator
-    rng = RanluxGenerator(queue, seed=15)
+    from pyopencl.clrandom import PhiloxGenerator
+    rng = PhiloxGenerator(context, seed=15)
 
     from time import time
 
@@ -865,7 +875,7 @@ def test_sort(ctx_factory, scan_kernel):
 
         dev_start = time()
         print("  device")
-        (a_dev_sorted,), evt = sort(a_dev, key_bits=16)
+        (a_dev_sorted,), _evt = sort(a_dev, key_bits=16)
         queue.finish()
         dev_end = time()
         print("  numpy")
@@ -905,7 +915,7 @@ def test_list_builder(ctx_factory):
             }
             """, arg_decls=[])
 
-    result, evt = builder(queue, 2000)
+    result, _evt = builder(queue, 2000)
 
     inf = result["mylist"]
     assert inf.count == 3000
@@ -930,7 +940,7 @@ def test_list_builder_with_memoryobject(ctx_factory):
 
     n = 10000
     input_list = cl.array.zeros(queue, (n,), float)
-    result, evt = builder(queue, n, input_list.data)
+    result, _evt = builder(queue, n, input_list.data)
 
     inf = result["mylist"]
     assert inf.count == n
@@ -958,7 +968,7 @@ def test_list_builder_with_offset(ctx_factory):
     input_list = cl.array.zeros(queue, (n + 10,), float)
     input_list[10:] = 1
 
-    result, evt = builder(queue, n, input_list[10:])
+    result, _evt = builder(queue, n, input_list[10:])
 
     inf = result["mylist"]
     assert inf.count == n
@@ -994,7 +1004,7 @@ def test_list_builder_with_empty_elim(ctx_factory):
         arg_decls=[],
         eliminate_empty_output_lists=["mylist1", "mylist2"])
 
-    result, evt = builder(queue, 1000)
+    result, _evt = builder(queue, 1000)
 
     mylist1 = result["mylist1"]
     assert mylist1.count == 19900
@@ -1027,7 +1037,7 @@ def test_key_value_sorter(ctx_factory):
 
     from pyopencl.algorithm import KeyValueSorter
     kvs = KeyValueSorter(context)
-    starts, lists, evt = kvs(queue, keys, values, nkeys, starts_dtype=np.int32)
+    starts, lists, _evt = kvs(queue, keys, values, nkeys, starts_dtype=np.int32)
 
     starts = starts.get()
     lists = lists.get()
@@ -1057,7 +1067,7 @@ def test_key_value_sorter(ctx_factory):
     ])
 @pytest.mark.bitonic
 def test_bitonic_sort(ctx_factory, size, dtype):
-    ctx = cl.create_some_context()
+    ctx = ctx_factory()
     queue = cl.CommandQueue(ctx)
 
     dev = ctx.devices[0]
@@ -1086,7 +1096,7 @@ def test_bitonic_sort(ctx_factory, size, dtype):
             and cl.get_cl_header_version() < (1, 2)):
         sgs.finish()
     sorter = BitonicSort(ctx)
-    sgs, evt = sorter(sgs, axis=1)
+    sgs, _evt = sorter(sgs, axis=1)
     assert np.array_equal(np.sort(s.get(), axis=1), sgs.get())
 
 
@@ -1111,7 +1121,7 @@ def test_bitonic_argsort(ctx_factory, size, dtype):
         pytest.xfail("pypy doesn't seem to handle as_strided "
                 "on zero-sized arrays very well")
 
-    ctx = cl.create_some_context()
+    ctx = ctx_factory()
     queue = cl.CommandQueue(ctx)
 
     device = queue.device
@@ -1155,7 +1165,7 @@ def test_bitonic_argsort(ctx_factory, size, dtype):
             and cl.get_cl_header_version() < (1, 2)):
         ms.finish()
         index.finish()
-    ms, evt = sorterm(ms, idx=index, axis=0)
+    ms, _evt = sorterm(ms, idx=index, axis=0)
 
     assert np.array_equal(np.sort(m.get()), ms.get())
 
