@@ -22,24 +22,25 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 
-import numpy as np
-import numpy.linalg as la
+import operator
+import platform
 import sys
 from itertools import product
 
+import numpy as np
+import numpy.linalg as la
 import pytest
 
 import pyopencl as cl
 import pyopencl.array as cl_array
 import pyopencl.cltypes as cltypes
 import pyopencl.tools as cl_tools
-from pyopencl.tools import (  # noqa: F401
-        pytest_generate_tests_for_pyopencl as pytest_generate_tests)
 from pyopencl.characterize import has_double_support, has_struct_arg_count_bug
+from pyopencl.clrandom import PhiloxGenerator, ThreefryGenerator
+from pyopencl.tools import (
+    pytest_generate_tests_for_pyopencl as pytest_generate_tests,  # noqa: F401
+)
 
-from pyopencl.clrandom import RanluxGenerator, PhiloxGenerator, ThreefryGenerator
-import operator
-import platform
 
 _PYPY = cl._PYPY
 
@@ -111,7 +112,7 @@ def test_mix_complex(ctx_factory):
 
     dtypes = [
             (np.float32, np.complex64),
-            #(np.int32, np.complex64),
+            # (np.int32, np.complex64),
             ]
 
     dev = context.devices[0]
@@ -302,7 +303,7 @@ def test_custom_type_zeros(ctx_factory):
     from pyopencl.tools import get_or_register_dtype, match_dtype_to_c_struct
 
     name = "mmc_type"
-    dtype, c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
+    dtype, _c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
     dtype = get_or_register_dtype(name, dtype)
 
     n = 1000
@@ -334,7 +335,7 @@ def test_custom_type_fill(ctx_factory):
     from pyopencl.tools import get_or_register_dtype, match_dtype_to_c_struct
 
     name = "mmc_type"
-    dtype, c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
+    dtype, _c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
     dtype = get_or_register_dtype(name, dtype)
 
     n = 1000
@@ -362,7 +363,7 @@ def test_custom_type_take_put(ctx_factory):
     from pyopencl.tools import get_or_register_dtype, match_dtype_to_c_struct
 
     name = "tp_type"
-    dtype, c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
+    dtype, _c_decl = match_dtype_to_c_struct(queue.device, name, dtype)
     dtype = get_or_register_dtype(name, dtype)
 
     n = 100
@@ -388,8 +389,8 @@ def test_custom_type_take_put(ctx_factory):
 # {{{ test_div_type_matches_numpy
 
 @pytest.mark.parametrize("dtype", [np.int8, np.int32, np.int64, np.float32])
-# FIXME Implement florodiv
-#@pytest.mark.parametrize("op", [operator.truediv, operator.floordiv])
+# FIXME Implement floordiv
+# @pytest.mark.parametrize("op", [operator.truediv, operator.floordiv])
 @pytest.mark.parametrize("op", [operator.truediv])
 def test_div_type_matches_numpy(ctx_factory, dtype, op):
     context = ctx_factory()
@@ -546,7 +547,7 @@ def test_addition_scalar(ctx_factory):
             ])
 def test_subtract_array(ctx_factory, dtype_a, dtype_b):
     """Test the subtraction of two arrays."""
-    #test data
+    # test data
     a = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]).astype(dtype_a)
     b = np.array([10, 20, 30, 40, 50,
                   60, 70, 80, 90, 100]).astype(dtype_b)
@@ -574,10 +575,10 @@ def test_subtract_scalar(ctx_factory):
     context = ctx_factory()
     queue = cl.CommandQueue(context)
 
-    #test data
+    # test data
     a = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10]).astype(np.float32)
 
-    #convert a to a gpu object
+    # convert a to a gpu object
     a_gpu = cl_array.to_device(queue, a)
 
     result = (a_gpu - 7).get()
@@ -605,7 +606,7 @@ def test_divide_scalar(ctx_factory):
                   np.float32, np.complex64)
     from pyopencl.characterize import has_double_support
     if has_double_support(queue.device):
-        dtypes = dtypes + (np.float64, np.complex128)
+        dtypes = (*dtypes, np.float64, np.complex128)
 
     from itertools import product
 
@@ -638,7 +639,7 @@ def test_divide_array(ctx_factory):
     dtypes = (np.float32, np.complex64)
     from pyopencl.characterize import has_double_support
     if has_double_support(queue.device):
-        dtypes = dtypes + (np.float64, np.complex128)
+        dtypes = (*dtypes, np.float64, np.complex128)
 
     from itertools import product
 
@@ -678,12 +679,11 @@ def test_divide_inplace_scalar(ctx_factory):
                   np.float32, np.complex64)
     from pyopencl.characterize import has_double_support
     if has_double_support(queue.device):
-        dtypes = dtypes + (np.float64, np.complex128)
+        dtypes = (*dtypes, np.float64, np.complex128)
 
     from itertools import product
 
     for dtype_a, dtype_s in product(dtypes, repeat=2):
-
         a = np.array([10, 20, 30, 40, 50, 60, 70, 80, 90, 100]).astype(dtype_a)
         s = dtype_s(40)
         a_gpu = cl_array.to_device(queue, a)
@@ -715,7 +715,7 @@ def test_divide_inplace_array(ctx_factory):
                   np.float32, np.complex64)
     from pyopencl.characterize import has_double_support
     if has_double_support(queue.device):
-        dtypes = dtypes + (np.float64, np.complex128)
+        dtypes = (*dtypes, np.float64, np.complex128)
 
     from itertools import product
 
@@ -785,15 +785,23 @@ def test_bitwise(ctx_factory):
 
             assert (res_dev.get() == res).all()
 
-            res_dev = op(a_dev, s)
-            res = op(a, s)
+            try:
+                res = op(a, s)
+            except OverflowError:
+                pass
+            else:
+                res_dev = op(a_dev, s)
 
-            assert (res_dev.get() == res).all()
+                assert (res_dev.get() == res).all()
 
-            res_dev = op(s, b_dev)
-            res = op(s, b)
+            try:
+                res = op(s, b)
+            except OverflowError:
+                pass
+            else:
+                res_dev = op(s, b_dev)
 
-            assert (res_dev.get() == res).all()
+                assert (res_dev.get() == res).all()
 
         for op in [o.iand, o.ior, o.ixor]:
             res_dev = a_dev.copy()
@@ -801,17 +809,25 @@ def test_bitwise(ctx_factory):
             assert op_res is res_dev
 
             res = a.copy()
-            op(res, b)
+            try:
+                op(res, b)
+            except OverflowError:
+                pass
+            else:
 
-            assert (res_dev.get() == res).all()
+                assert (res_dev.get() == res).all()
 
-            res_dev = a_dev.copy()
-            op_res = op(res_dev, s)
-            assert op_res is res_dev
             res = a.copy()
-            op(res, s)
+            try:
+                op(res, s)
+            except OverflowError:
+                pass
+            else:
+                res_dev = a_dev.copy()
+                op_res = op(res_dev, s)
+                assert op_res is res_dev
 
-            assert (res_dev.get() == res).all()
+                assert (res_dev.get() == res).all()
 
         # Test unary ~
         res_dev = ~a_dev
@@ -828,33 +844,18 @@ def test_bitwise(ctx_factory):
 # {{{ test_random_float_in_range
 
 @pytest.mark.parametrize("rng_class",
-        [RanluxGenerator, PhiloxGenerator, ThreefryGenerator])
+        [PhiloxGenerator, ThreefryGenerator])
 @pytest.mark.parametrize("ary_size", [300, 301, 302, 303, 10007, 1000000])
 def test_random_float_in_range(ctx_factory, rng_class, ary_size, plot_hist=False):
     context = ctx_factory()
     queue = cl.CommandQueue(context)
 
-    device = queue.device
-    if device.platform.vendor == "The pocl project" \
-            and device.type & cl.device_type.GPU \
-            and rng_class is RanluxGenerator:
-        pytest.xfail("ranlux test fails on PoCL + Nvidia,"
-                "at least the Titan V, as of PoCL 1.6, 2021-01-20")
-
-    if device.platform.vendor == "Intel(R) Corporation" \
-            and rng_class is RanluxGenerator:
-        pytest.xfail("compiling ranlux kernel causes a segfault on "
-                "Intel CPU runtime as of 2022-02-13")
-
     if has_double_support(context.devices[0]):
         dtypes = [np.float32, np.float64]
     else:
         dtypes = [np.float32]
 
-    if rng_class is RanluxGenerator:
-        gen = rng_class(queue, 5120)
-    else:
-        gen = rng_class(context)
+    gen = rng_class(context)
 
     for dtype in dtypes:
         print(dtype)
@@ -869,9 +870,6 @@ def test_random_float_in_range(ctx_factory, rng_class, ary_size, plot_hist=False
         assert (0 <= ran.get()).all()
         assert (ran.get() <= 1).all()
 
-        if rng_class is RanluxGenerator:
-            gen.synchronize(queue)
-
         ran = cl_array.zeros(queue, ary_size, dtype)
         gen.fill_uniform(ran, a=4, b=7)
         ran_host = ran.get()
@@ -897,21 +895,12 @@ def test_random_float_in_range(ctx_factory, rng_class, ary_size, plot_hist=False
 
 @pytest.mark.parametrize("dtype", [np.int32, np.int64])
 @pytest.mark.parametrize("rng_class",
-        [RanluxGenerator, PhiloxGenerator, ThreefryGenerator])
+        [PhiloxGenerator, ThreefryGenerator])
 def test_random_int_in_range(ctx_factory, rng_class, dtype, plot_hist=False):
     context = ctx_factory()
     queue = cl.CommandQueue(context)
 
-    if queue.device.platform.vendor == "The pocl project" \
-            and queue.device.type & cl.device_type.GPU \
-            and rng_class is RanluxGenerator:
-        pytest.xfail("ranlux test fails on PoCL + Nvidia,"
-                "at least the Titan V, as of PoCL 1.6, 2021-01-20")
-
-    if rng_class is RanluxGenerator:
-        gen = rng_class(queue, 5120)
-    else:
-        gen = rng_class(context)
+    gen = rng_class(context)
 
     # if (dtype == np.int64
     #         and context.devices[0].platform.vendor.startswith("Advanced Micro")):
@@ -1158,12 +1147,12 @@ def test_slice(ctx_factory):
     a = a_gpu.get()
     b = b_gpu.get()
 
+    start_offset = 0
+
     if queue.device.platform.name == "Intel(R) OpenCL":
         pytest.skip("Intel CL regularly crashes on this test case "
             "-- https://github.com/conda-forge/"
             "intel-compiler-repack-feedstock/issues/7")
-    else:
-        start_offset = 0
 
     from random import randrange
     for _i in range(20):
@@ -1177,7 +1166,7 @@ def test_slice(ctx_factory):
 
     for _i in range(20):
         start = randrange(ary_len-start_offset)
-        #end = randrange(start+start_offset, ary_len)
+        # end = randrange(start+start_offset, ary_len)
         end = start
 
         a_gpu[start:end] = tp(2)*b[start:end]
@@ -1477,7 +1466,7 @@ def test_transpose(ctx_factory):
     a = a_gpu.get()
 
     # FIXME: not contiguous
-    #assert np.allclose(a_gpu.transpose((1,2,0)).get(), a.transpose((1,2,0)))
+    # assert np.allclose(a_gpu.transpose((1,2,0)).get(), a.transpose((1,2,0)))
     assert np.array_equal(a_gpu.T.get(), a.T)
 
 # }}}
@@ -1527,7 +1516,7 @@ def test_squeeze(ctx_factory):
     assert a_gpu_squeezed_slice.flags.c_contiguous
 
     # Check that we get the original values out
-    #assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
+    # assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
 
     # Slice with length 1 on dimensions 2
     a_gpu_slice = a_gpu[:, :, 2:3, :]
@@ -1540,7 +1529,7 @@ def test_squeeze(ctx_factory):
     assert not a_gpu_squeezed_slice.flags.c_contiguous
 
     # Check that we get the original values out
-    #assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
+    # assert np.all(a_gpu_slice.get().ravel() == a_gpu_squeezed_slice.get().ravel())
 
 # }}}
 
@@ -1649,7 +1638,8 @@ def test_get_async(ctx_factory):
     b_gpu = a_gpu + a_gpu**5 + 1
 
     # deprecated, but still test
-    b1 = b_gpu.get(async_=True)  # testing that this waits for events
+    with pytest.deprecated_call():
+        b1 = b_gpu.get(async_=True)  # testing that this waits for events
     b_gpu.finish()
     assert np.abs(b1 - b).mean() < 1e-5
 
@@ -2346,10 +2336,81 @@ def test_logical_not(ctx_factory):
         cl_array.logical_not(cl_array.zeros(cq, 10, np.float64)).get(),
         np.logical_not(np.zeros(10)))
     np.testing.assert_array_equal(
-        cl_array.logical_not((cl_array.zeros(cq, 10, np.float64) + 1)).get(),
+        cl_array.logical_not(cl_array.zeros(cq, 10, np.float64) + 1).get(),
         np.logical_not(np.ones(10)))
 
 
+# {{{ test XDG_CACHE_HOME handling
+
+@pytest.mark.skipif(sys.platform == "win32",
+                    reason="XDG_CACHE_HOME is not used on Windows")
+def test_xdg_cache_home(ctx_factory):
+    import os
+    import shutil
+    from os.path import join
+
+    context = ctx_factory()
+    queue = cl.CommandQueue(context)
+
+    a = np.array([1, 2, 3, 4, 5]).astype(np.float32)
+    a_gpu = cl_array.to_device(queue, a)
+
+    xdg_dir = "tmpdir_pyopencl_xdg_test"
+
+    # PyOpenCL uses pytools.PersistentDict for invoker caches,
+    # which is why xdg_dir will always exist. Therefore, check
+    # whether xdg_pyopencl_dir exists.
+    xdg_pyopencl_dir = join(xdg_dir, "pyopencl")
+    assert not os.path.exists(xdg_dir)
+
+    old_xdg_cache_home = None
+    old_characterize_has_src_build_cache = None
+
+    try:
+        # Make sure that the source build cache is enabled
+        old_characterize_has_src_build_cache = \
+            cl.characterize.has_src_build_cache
+        cl.characterize.has_src_build_cache = lambda dev: False
+
+        old_xdg_cache_home = os.getenv("XDG_CACHE_HOME")
+        os.environ["XDG_CACHE_HOME"] = xdg_dir
+
+        result = pow(a_gpu, a_gpu).get()
+        assert (np.abs(a ** a - result) < 3e-3).all()
+
+        assert os.path.exists(xdg_pyopencl_dir)
+    finally:
+        cl.characterize.has_src_build_cache = \
+            old_characterize_has_src_build_cache
+
+        if old_xdg_cache_home is not None:
+            os.environ["XDG_CACHE_HOME"] = old_xdg_cache_home
+        else:
+            del os.environ["XDG_CACHE_HOME"]
+
+        shutil.rmtree(xdg_dir)
+
+# }}}
+
+
+def test_numpy_type_promotion_with_cl_arrays(ctx_factory):
+    ctx = ctx_factory()
+    queue = cl.CommandQueue(ctx)
+
+    class NotReallyAnArray:
+        @property
+        def dtype(self):
+            return np.dtype("float64")
+
+    # Make sure that np.result_type accesses only the dtype attribute of the
+    # class, not (e.g.) its data.
+    assert np.result_type(42, NotReallyAnArray()) == np.float64
+
+    from pyopencl.array import _get_common_dtype
+    assert _get_common_dtype(42, NotReallyAnArray(), queue) == np.float64
+    assert _get_common_dtype(42.0, NotReallyAnArray(), queue) == np.float64
+
+
 if __name__ == "__main__":
     if len(sys.argv) > 1:
         exec(sys.argv[1])
