@@ -26,12 +26,14 @@ import pytest
 
 import pyopencl as cl
 import pyopencl.array as cl_array
-import pyopencl.cltypes as cltypes
 import pyopencl.clrandom
-from pyopencl.tools import (  # noqa: F401
-        pytest_generate_tests_for_pyopencl as pytest_generate_tests,
-        ImmediateAllocator, DeferredAllocator)
+import pyopencl.cltypes as cltypes
 from pyopencl.characterize import get_pocl_version
+from pyopencl.tools import (
+    DeferredAllocator,
+    ImmediateAllocator,
+    pytest_generate_tests_for_pyopencl as pytest_generate_tests,  # noqa: F401
+)
 
 
 def _xfail_if_pocl(plat, up_to_version, msg="unsupported by PoCL"):
@@ -54,8 +56,9 @@ def test_get_info(ctx_factory):
     device, = ctx.devices
     platform = device.platform
 
-    device.persistent_unique_id
-    device.hashable_model_and_version_identifier
+    with pytest.deprecated_call():
+        device.persistent_unique_id  # noqa: B018
+    device.hashable_model_and_version_identifier  # noqa: B018
 
     failure_count = [0]
 
@@ -163,6 +166,9 @@ def test_get_info(ctx_factory):
             & cl.command_queue_properties.PROFILING_ENABLE):
         profiling = True
         props = cl.command_queue_properties.PROFILING_ENABLE
+    else:
+        profiling = False
+
     queue = cl.CommandQueue(ctx,
             properties=props)
     do_test(queue, cl.command_queue_info)
@@ -221,8 +227,8 @@ def test_get_info(ctx_factory):
         img = cl.Image(ctx, cl.mem_flags.READ_ONLY, img_format, (128, 256))
         assert img.shape == (128, 256)
 
-        img.depth
-        img.image.depth
+        img.depth  # noqa: B018
+        img.image.depth  # noqa: B018
         do_test(img, cl.image_info,
                 lambda info: img.get_image_info(info))
 
@@ -293,7 +299,7 @@ def test_invalid_kernel_names_cause_failures(ctx_factory):
         """).build()
 
     try:
-        prg.sam
+        prg.sam  # noqa: B018
         raise RuntimeError("invalid kernel name did not cause error")
     except AttributeError:
         pass
@@ -476,7 +482,7 @@ def test_image_2d(ctx_factory):
 # {{{ test_image_3d
 
 def test_image_3d(ctx_factory):
-    #test for image_from_array for 3d image of float2
+    # test for image_from_array for 3d image of float2
     context = ctx_factory()
 
     device, = context.devices
@@ -516,7 +522,7 @@ def test_image_3d(ctx_factory):
     shape = (3, 4, 2)
 
     rng = np.random.default_rng(seed=42)
-    a = rng.random(size=shape + (num_channels,), dtype=np.float32)
+    a = rng.random(size=(*shape, num_channels), dtype=np.float32)
 
     queue = cl.CommandQueue(context)
     try:
@@ -581,7 +587,7 @@ def test_copy_buffer(ctx_factory):
 # {{{ test_mempool_*
 
 def test_mempool(ctx_factory):
-    from pyopencl.tools import MemoryPool, ImmediateAllocator
+    from pyopencl.tools import ImmediateAllocator, MemoryPool
 
     context = ctx_factory()
     queue = cl.CommandQueue(context)
@@ -601,9 +607,10 @@ def test_mempool(ctx_factory):
 
 
 def test_mempool_2(ctx_factory):
-    from pyopencl.tools import MemoryPool, ImmediateAllocator
     from random import randrange
 
+    from pyopencl.tools import ImmediateAllocator, MemoryPool
+
     context = ctx_factory()
     queue = cl.CommandQueue(context)
 
@@ -684,7 +691,7 @@ def test_vector_args(ctx_factory):
 def test_header_dep_handling(ctx_factory):
     context = ctx_factory()
 
-    from os.path import exists, dirname, join
+    from os.path import dirname, exists, join
     assert exists(join(dirname(__file__), "empty-header.h"))
 
     kernel_src = """
@@ -858,22 +865,22 @@ def test_user_event(ctx_factory):
     evt = cl.UserEvent(ctx)
     Thread(target=event_waiter1, args=(evt, 1)).start()
     sleep(.05)
-    if status.get(1, False):
+    if status.get(1):
         raise RuntimeError("UserEvent triggered before set_status")
     evt.set_status(cl.command_execution_status.COMPLETE)
     sleep(.05)
-    if not status.get(1, False):
+    if not status.get(1):
         raise RuntimeError("UserEvent.wait timeout")
     assert evt.command_execution_status == cl.command_execution_status.COMPLETE
 
     evt = cl.UserEvent(ctx)
     Thread(target=event_waiter2, args=(evt, 2)).start()
     sleep(.05)
-    if status.get(2, False):
+    if status.get(2):
         raise RuntimeError("UserEvent triggered before set_status")
     evt.set_status(cl.command_execution_status.COMPLETE)
     sleep(.05)
-    if not status.get(2, False):
+    if not status.get(2):
         raise RuntimeError("cl.wait_for_events timeout on UserEvent")
     assert evt.command_execution_status == cl.command_execution_status.COMPLETE
 
@@ -1124,8 +1131,9 @@ def test_coarse_grain_svm(ctx_factory, use_opaque_style):
 
     dev = ctx.devices[0]
 
-    from pyopencl.characterize import has_coarse_grain_buffer_svm
     from pytest import skip
+
+    from pyopencl.characterize import has_coarse_grain_buffer_svm
     if not has_coarse_grain_buffer_svm(queue.device):
         skip("device does not support coarse-grain SVM")
 
@@ -1232,8 +1240,9 @@ def test_fine_grain_svm(ctx_factory):
 
     _xfail_if_pocl_gpu(queue.device, "GPU SVM")
 
-    from pyopencl.characterize import has_fine_grain_buffer_svm
     from pytest import skip
+
+    from pyopencl.characterize import has_fine_grain_buffer_svm
     if not has_fine_grain_buffer_svm(queue.device):
         skip("device does not support fine-grain SVM")
 
@@ -1282,7 +1291,7 @@ def test_map_dtype(ctx_factory, dtype):
     b = pyopencl.Buffer(ctx,
                         pyopencl.mem_flags.READ_ONLY,
                         dt.itemsize)
-    array, ev = pyopencl.enqueue_map_buffer(queue, b, pyopencl.map_flags.WRITE, 0,
+    array, _ev = pyopencl.enqueue_map_buffer(queue, b, pyopencl.map_flags.WRITE, 0,
                                             (1,), dt)
     with array.base:
         print(array.dtype)
@@ -1393,7 +1402,7 @@ def test_empty_ndrange(ctx_factory, empty_shape):
     queue = cl.CommandQueue(ctx)
 
     if ctx._get_cl_version() < (1, 2) or cl.get_cl_header_version() < (1, 2):
-        pytest.skip("OpenCL 1.2 required for empty NDRange suuport")
+        pytest.skip("OpenCL 1.2 required for empty NDRange support")
 
     a = cl_array.zeros(queue, empty_shape, dtype=np.float32)
 
@@ -1428,8 +1437,9 @@ def test_capture_call(ctx_factory):
     ctx = ctx_factory()
     queue = cl.CommandQueue(ctx)
 
-    a_np = np.random.rand(500).astype(np.float32)
-    b_np = np.random.rand(500).astype(np.float32)
+    rng = np.random.default_rng()
+    a_np = rng.random(500, dtype=np.float32)
+    b_np = rng.random(500, dtype=np.float32)
 
     ctx = cl.create_some_context()
     queue = cl.CommandQueue(ctx)
@@ -1514,6 +1524,42 @@ def test_enqueue_copy_array_2(ctx_factory):
 # }}}
 
 
+def test_zero_size_svm_allocations(ctx_factory):
+    ctx = ctx_factory()
+
+    from pytest import skip
+
+    from pyopencl.characterize import has_coarse_grain_buffer_svm
+    if not has_coarse_grain_buffer_svm(ctx.devices[0]):
+        skip("device does not support coarse-grain SVM")
+
+    # Go back to svm_empty once
+    # https://github.com/numpy/numpy/issues/26366 is solved.
+    # zero_sized_svm = cl.svm_empty(ctx, cl.svm_mem_flags.READ_WRITE, 0, np.float64)
+    zero_sized_svm = cl.SVMAllocation(ctx, 0, 0, cl.svm_mem_flags.READ_WRITE)
+    zero_sized_svm.release()
+
+    from pyopencl.tools import SVMAllocator, SVMPool
+    svm_alloc = SVMAllocator(ctx)
+    zero_sized_svm = svm_alloc(0)
+    zero_sized_svm.release()
+
+    svm_pool = SVMPool(svm_alloc)
+    zero_sized_svm = svm_pool(0)
+    zero_sized_svm.release()
+
+
+def test_buffer_release(ctx_factory):
+    ctx = ctx_factory()
+    queue = cl.CommandQueue(ctx)
+
+    mem_pool = cl.tools.MemoryPool(cl.tools.ImmediateAllocator(queue))
+
+    b = mem_pool.allocate(1000)
+    print(type(b))
+    b.release()
+
+
 if __name__ == "__main__":
     import sys
     if len(sys.argv) > 1:
